#include <sys/types.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include "license-base.h"

/* Like strncpy but make sure the resulting string is always 0 terminated. */
static char *safe_strncpy(char *dst, const char *src, size_t size)
{
	dst[size-1] = '\0';
	return strncpy(dst,src,size-1);
}

/* like strcmp(), but knows about numbers */
static int nstrcmp(const char *astr, const char *b)
{
	const char *a = astr;

	while (*a == *b) {
		if (*a == '\0') return 0;
		a++;
		b++;
	}

	if (isdigit(*a)) {
		if (!isdigit(*b))
			return 1;
		while (a > astr) {
			a--;
			if (!isdigit(*a)) {
				a++;
				break;
			}

			if (!isdigit(*b)) return -1;
				b--;
        	}

        	return atoi(a) > atoi(b) ? 1 : -1;
	}

	return *a - *b;
}


static char *trimWhitespace(char *str)
{
    char *end;
    /* Trim leading space */
    while(isspace(*str)) str++;

    /* Trim trailing space */
    end = str + strlen(str) - 1;
    while(end > str && isspace(*end)) end--;

    /* Write new null terminator */
    *(end+1) = 0;

    return str;
}

static int chk_digit_str(char *str)
{
  char *h = str;
  char c;
  while((c = *h++) != '\0') {
    if(!isdigit(c)){
      return -1;
    }
  }
  return 0;
}

static char *MytoLower(char *src)
{
  char *p = src;
  while(*p != '\0')
  {
    if(64 < *p && *p < 91)//a:65，z:90
    {
      *p = (*p+32);
    }
    p++;
  }
  return src;
}
 
static int chk_strin_vtlist(char *machine_str)
{
  static char *vtlist[] = {"kvm", "vmware", NULL};
 
  char *str = machine_str;
  char **vt_ptr = NULL;

  MytoLower(str);
  for(vt_ptr = vtlist; *vt_ptr != NULL; vt_ptr++) {
    if(strstr(str, *vt_ptr)) { /*in the vt list*/
      return 1;
    }
  } 
  return 0;
}

static int run_command(char *cmd_str, char *cmd_print_out, int *pclose_retval, int *cmd_retval)
{
  FILE *fp;
  char temp[1035];
  memset(temp,0,1035);
  int rc=0;
  fp=popen(cmd_str,"r");

  if (fp == NULL) {
#ifdef LICENSE_LIB_DEBUG
    printf("popen dmidecode error\n");
#endif
    return -1;
  }

  while (fgets(temp, sizeof(temp)-1, fp) != NULL) {
    if(temp[0]!=0) {
      if (temp[strlen(temp) - 1] == '\n') {
        temp[strlen(temp) - 1] = '\0'; //去除换行符
      }
      strcpy(cmd_print_out, temp);
    }
  }
    /*等待命令执行完毕并关闭管道及文件指针*/
    rc = pclose(fp);
    if(-1 == rc)
    {
#ifdef LICENSE_LIB_DEBUG
        perror("failed to close fp.\n");
#endif
        return -1;
    }
    else
    {
        *pclose_retval = rc;
        *cmd_retval = WEXITSTATUS(rc);
#ifdef LICENSE_LIB_DEBUG
        printf("command【%s】subprocess finish status【%d】, command return value【%d】\r\n", cmd_str, rc, WEXITSTATUS(rc));
	if (strlen(cmd_print_out) > 0) {
        	printf("command output:%s\n", cmd_print_out);
	}
#endif
        return 0;
    }
  return 0;

}

static int run_get_client_cpu(char *cmd_str, int *cpu_num) {
  int snum;
  char cmdoutput[64];
  memset(cmdoutput, 0, 64);
  int rc, cmd_ret, pclose_ret;
  int retval = -1;
  rc = run_command(cmd_str, cmdoutput, &pclose_ret, &cmd_ret);  
#ifdef LICENSE_LIB_DEBUG
  printf("pcolse return value:%d, cmd return value:%d\n", pclose_ret, cmd_ret);
#endif

  if (!rc && !cmd_ret && !pclose_ret) {  /* success */
    char *toutput = trimWhitespace(cmdoutput);
    if((strlen(toutput) > 0) && (chk_digit_str(toutput) == 0)) {
      snum = atoi(toutput);
      if (snum < 99) {
        *cpu_num = snum;
        retval = 0;
      } 
    }
  }
  return retval;
}

/*int get_client_cpusocket(int *cpu_num) {
  int ret_cmd;
  int ret_getcpu = 0;
  char cmd[128];
  memset(cmd,0,128);
  sprintf(cmd, "dmidecode -t4 > /dev/null 2>&1 && dmidecode -t4 | grep Socket.Designation: | wc -l");
  ret_cmd = run_get_client_cpu(cmd, cpu_num);

  if (ret_cmd != 0) {
    memset(cmd,0,128);
    sprintf(cmd, "lscpu > /dev/null 2>&1 && lscpu -p=socket | grep -v ^# | sort |uniq | wc -l");
    ret_cmd = run_get_client_cpu(cmd, cpu_num);
    if (ret_cmd != 0) {
      memset(cmd,0,128);
      sprintf(cmd, "grep 'physical id' /proc/cpuinfo | awk -F: '{print $2 | \"sort -un\"}' | wc -l");
      ret_cmd = run_get_client_cpu(cmd, cpu_num);
      if (ret_cmd != 0) {
        ret_getcpu = -1;
      }
    }
  }
  return ret_getcpu;
}*/

int get_client_cpusocket(int *cpu_num) {
  int ret_cmd;
  int ret_getcpu = 0;
  char cmd[128];
  memset(cmd,0,128);
  sprintf(cmd, "dmidecode -t4 > /dev/null 2>&1 && dmidecode -t4 | grep \"Status: Populated,\" | wc -l");
  ret_cmd = run_get_client_cpu(cmd, cpu_num);

  if (ret_cmd != 0 || *cpu_num == 0) {
    memset(cmd,0,128);
    sprintf(cmd, "lscpu > /dev/null 2>&1 && LANG=C lscpu -p=node | grep -v ^# | sort |uniq | wc -l");
    ret_cmd = run_get_client_cpu(cmd, cpu_num);
    if (ret_cmd != 0 || *cpu_num == 0) {
      memset(cmd,0,128);
      sprintf(cmd, "grep 'physical id' /proc/cpuinfo | sort | uniq | wc -l");
      ret_cmd = run_get_client_cpu(cmd, cpu_num);
      if (ret_cmd != 0 || *cpu_num == 0) {
        ret_getcpu = -1;
      }
    }
  }
  return ret_getcpu;
}

int get_client_machinetype(int *machine_type)
{
  char cmdoutput[64];
  memset(cmdoutput, 0, 64);
  int rc, cmd_ret, pclose_ret;
  int ret_getcpu, cpunum, ret_chkvt, invtlist;
  char cmd[128];
  memset(cmd,0,128);
  sprintf(cmd, "dmesg | grep \"Hypervisor detected\" > /dev/null 2>&1");
  rc = run_command(cmd, cmdoutput, &pclose_ret, &cmd_ret);
  if (!rc && !cmd_ret && !pclose_ret) {  /* success: dmesg grep Hypervisor command success, machinetype is VT.*/
    *machine_type=0;
    return 0;
  } else {  /* goto method2 */
    memset(cmdoutput, 0, 64);
    memset(cmd,0,128);
    sprintf(cmd, "virt-what");
    rc = run_command(cmd, cmdoutput, &pclose_ret, &cmd_ret);
    if (!rc && !cmd_ret && !pclose_ret) {  /* success: */
      /* virt-what command success, check cmdoutput length is zero? */
      if(strlen(cmdoutput) == 0) { /*this is not VT, then get number of CPU sockets*/
        ret_getcpu = get_client_cpusocket(&cpunum);
        if (ret_getcpu == 0) {
          *machine_type = cpunum;
	  return 0;
        } else {
          return -1;
        }
      }else{  /* this is VT. todo: may be virt-what error output*/
        *machine_type=0;
        return 0;
      }
    } else {
      memset(cmdoutput, 0, 64);
      memset(cmd,0,128);
      sprintf(cmd, "dmidecode -s system-product-name");
      rc = run_command(cmd, cmdoutput, &pclose_ret, &cmd_ret);
      if (rc || cmd_ret || pclose_ret || (strlen(cmdoutput) == 0)) { /*failed to run*/
        return -1;
      } else {
        /* check the cmdoutput is in the VT list*/
        ret_chkvt = chk_strin_vtlist(cmdoutput);
        if (ret_chkvt == 1){  /* cmdoutput is in the vt list */
          *machine_type=0;
          return 0;
        } else { /* cmdoutput is not vt, then get number of CPU sockets */
          ret_getcpu = get_client_cpusocket(&cpunum);
          if (ret_getcpu == 0) {
            *machine_type = cpunum;
	    return 0;
          } else {
            return -1;
          }
        }
      }
    } 
  }
}

