/**
* Copyright 2012 CS2C All Rights Reserved.
* Authors:WangLei <lei.wang@cs2c.com.cn>
* 
* this file is a header file for license server.
* the main function is to generate a license.
*/
#ifndef __LICENSE_BASE_H_
#define __LICENSE_BASE_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <openssl/evp.h>
#include <openssl/bio.h>

#pragma pack(1)

#define	CS2C_LICENSE_LIB_VERSION	"1.0-60"

/* error code defined here */
#define         CS2C_LICENSE_OK                 0x00000000              /* operating success */
#define         CS2C_LICENSE_PARA_ERR           0xE0000001              /* parameter error */
#define         CS2C_LICENSE_NO_USBKEY          0xE0000002              /* no usbkey which should be pluged in */
#define         CS2C_LICENSE_BUF_SMALL          0xE0000003              /* buffer is too small */
#define         CS2C_LICENSE_FEEDBACK_ERR       0xE0000004              /* feedback wrong */
#define         CS2C_LICENSE_SIG_ERR            0xE0000005              /* signature wrong */
#define		CS2c_LICENSE_MORE_USBKEY	0xE0000006		/* more than 1 ukey plugin */
#define		CS2C_LICENSE_USBKEY_NOLICENSE	0xE0000007		/* ukey is plugin but no license */
#define		CS2C_LICENSE_USBKEY_PKCSERR	0xE0000008		/* general pkcs11 error */
#define		CS2C_LICENSE_NO_PRIVATE_KEY	0xE0000009		/* no license private key */
#define		CS2C_LICENSE_NO_CERT		0xE000000A		/* no license cert */
#define 	CS2C_LICENSE_OPENSSL_ERR	0xE000000B		/* general openssl error */
#define 	CS2C_LICENSE_RSA_VERIFY_FAILED	0xE000000C		/* rsa verify failed */
#define 	CS2C_LICENSE_RSA_SIGN_FAILED	0xE000000D		/* rsa sign failed */
#define		CS2C_LICENSE_LICENSE_INFO_ERR	0xE000000E		/* license info error */
#define		CS2C_LICENSE_STRUCT_ERR		0xE0000010		/* license struct is wrong:iftry,feedback must be null */
#define		CS2C_LICENSE_FILE_ERR		0xE0000011		/* general file operation error */
//#define         CS2C_LICENSE_FEEDBACK_ERR   	0xE0000012              /* feedback format wrong */
#define		CS2C_LICENSE_FEEDBACK_GEN_ERR	0xE0000013		/* feedback generate error */
#define		CS2C_LICENSE_IS_EXPIRED		0xE0000014		/* license is expired */
#define		CS2C_LICENSE_NO_PID		0xE0000016		/* license file isn't contain the infomation of pid */
#define		CS2C_LICENSE_NEED_INSTALL	0xE0000017		/* license file needs to be installed */
#define		CS2C_LICENSE_DOUBLE_INSTALLED	0xE0000018		/* license file installed twice */
#define		CS2C_LICENSE_INVALID_FILE	0xE0000019		/* invalid license file error */
#define		CS2C_LICENSE_MIPS_NO_UKEY	0xE000001A		/* loongson platform don't support ukey */
#define		CS2C_LICENSE_PROC_READERR	0xE000001B		/* can not read /proc/net/dev */
#define		CS2C_LICENSE_SOCKETERR		0xE000001C		/* can not get socket AF_INET */
#define		CS2C_LICENSE_NO_USEFUL_NIC	0xE000001D		/* no useful net interface */

/*NeoKylin add 20160505*/
#define		FB_MAC_FIRST_SECTION		4			/* the first section of 16 md5 value of mac feedback*/
#define         FB_MAC_SENCOND_SECTION		8			/* the second section of 16 md5 value of mac feedback */
#define		FB_MACHINE_LENGTH		2			/* the length of machine type of feedback */
#define		FB_PRODUCTID_LENGTH		2			/* the length of product id of feedback */

#define 	PID_ADVAN_V7 		13
#define         PID_ADVAN_V7_MAP	'c'
#define 	PID_VIRTUAL_V7 		42       
#define         PID_VIRTUAL_V7_MAP	'd'
#define 	PID_GENER 		11
#define 	PID_GENER_MAP		'9'
#define 	PID_ADVAN 		12
#define 	PID_ADVAN_MAP		'a'
#define 	PID_VIRTUAL 		41
#define         PID_VIRTUAL_MAP		'b'
#define         PID_LIST_LENGTH         5

#define		FB_LENGTH		33
#define		FB_STRING_LENGTH	FB_LENGTH + 1
#define		FB_LENGTH_V2		37
#define		FB_STRING_LENGTH_V2	FB_LENGTH_V2 + 1
//int PRODUCTID_LIST[PID_LIST_LENGTH] = {PID_ADVAN_V7, PID_VIRTUAL_V7, PID_ADVAN, PID_VIRTUAL, PID_GENER};
//char PRODUCTIDID_MAP_LIST[PID_LIST_LENGTH] = {PID_ADVAN_V7_MAP, PID_VIRTUAL_V7_MAP, PID_ADVAN_MAP, PID_VIRTUAL_MAP, PID_GENER_MAP}; 

/* type defined here */
typedef unsigned char u8 ;
typedef unsigned short u16;
typedef unsigned int u32;


/* license header :which company,who and when issued the license */
typedef struct license_header {

        u16     header_length;
        u16     control_info_length;
	u16	feedback_length;
        u16     verify_length;

}cs2c_license_header;


/* the main control info */
/* pid:         product id.
 * vid:         version id.
 * node_num:    the number of nodes.
 * user_num:    the number of users.
 * modules:	modules number.
 * days:        how many days the user can use our product.
 * iftry:       1 means try,0 means no try.
 * installed:   date of installing license.
 */
typedef struct license_control_info {

        u8	pid;
        u8	vid;
        u16     user_num;
        u16     node_num;
	u32	modules;
        u32     days;
        u8      iftry;
        u8      installed[12];

}cs2c_license_control_info;

/* 
 * customer:	cumstomer infomation
 * feedback:	feedback code
 * expand:	expand infomation
 */
typedef struct license_feedback {

	char	customer[80];
	char	feedback[FB_STRING_LENGTH];
	char	expand[16];
	char	ifukey;

}cs2c_license_feedback;

/* license signiture infomation which concerns several things:
 * length:              signature length
 * sig                  signature strings.
 */
typedef struct license_signature{
        u8      length;
        u8*     sig;
} cs2c_license_signature;


/* license struct */
typedef struct license{
        cs2c_license_header             *header;
        cs2c_license_control_info       *control_info;
	cs2c_license_feedback		*feedback;
        cs2c_license_signature          *signature;
} cs2c_license;



#pragma pack()

/*
int get_license_ukeyflag(cs2c_license_feedback *feedback, int *ukey_flag);
int get_license_tryflag(cs2c_license_control_info *control_info, int product_number, int *try_flag);
int base64(u8 *in, u32 ilen, u8 *out, u32 *olen);
int debase64(u8 *in, u32 ilen,u8 *out, u32 *olen);
int print_license_control_info(cs2c_license_control_info *control_info);
int print_license(cs2c_license *license);
int free_license_struct(cs2c_license *license);
int print_license_err(int rv);
int get_lib_version(char *v,int *vlen);
*/


#endif

