#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <openssl/md5.h>
#include <linux/hdreg.h>
#include <fcntl.h>

#include "license-client.h"
#include "consts.h"
#include "usbkey.h"
#include "log.h"
#include "mac.h"
#include "client-machinetype.h"

int PRODUCTID_LIST[PID_LIST_LENGTH] = {PID_ADVAN_V7, PID_VIRTUAL_V7, PID_ADVAN, PID_VIRTUAL, PID_GENER};
char PRODUCTIDID_MAP_LIST[PID_LIST_LENGTH] = {PID_ADVAN_V7_MAP, PID_VIRTUAL_V7_MAP, PID_ADVAN_MAP, PID_VIRTUAL_MAP, PID_GENER_MAP};

static int get_hd_sn(const char* dev, char* sn, int *slen)
{
	struct hd_driveid id;
	int  rv = -1;

	int  fd = open(dev, O_RDONLY|O_NONBLOCK);
	while(1)
	{
        	if (fd < 0)
        	{
			perror(dev);
		        break;
        	}

        	if(!ioctl(fd, HDIO_GET_IDENTITY, &id))
        	{
             		strncpy(sn, id.model, strlen(id.model));
             		strncpy(sn+strlen(id.model), id.serial_no, strlen(id.serial_no));
#ifdef LICENSE_LIB_DEBUG
             		printf("Model=%s\n",id.model);
             		printf("SN=%s\n",id.serial_no);
#endif
             		*slen = strlen(id.serial_no)+strlen(id.model);
             		rv = 0;
        	}
        	break;
    	}
	
//	printf("sn = %s,slen= \n",sn,*slen);
	return rv;
}



static int fetch_hardisk_info(char *hdisk_info, int *len)
{
	int rv = 0;
	
	rv = get_hd_sn("/dev/sda",hdisk_info,len);
	if(rv != 0)
		return CS2C_LICENSE_FEEDBACK_GEN_ERR;
	return 0;
}


int mkmd5(const u8 *buf, u32 len,
                        u8 *mdv,int *mdvlen)
{
        EVP_MD_CTX ctx;
        const EVP_MD *md;

        OpenSSL_add_all_algorithms();
        ERR_load_crypto_strings();

        OpenSSL_add_all_digests();
        md = EVP_get_digestbyname("md5");

        EVP_MD_CTX_init(&ctx);
        EVP_DigestInit_ex(&ctx,md,NULL);
        EVP_DigestUpdate(&ctx,buf,len);
        EVP_DigestFinal_ex(&ctx,mdv,mdvlen);
        EVP_MD_CTX_cleanup(&ctx);

        return 0;
}


static int license_verify_from_license_struct(cs2c_license *license)
{
        int rv = 0;
        char feedback[34];
        int length = 34;
	int fbtype = 0;
	int iftry = 0;
	int product_number = 0;
	int i = 0;
	char hash32[32];

	memset(feedback,0,34);
	fbtype = license->feedback->feedback[0]-0x30;
	product_number = license->header->control_info_length/sizeof(cs2c_license_control_info);	

	for(i=0;i<product_number;i++)
		iftry |= license->control_info[i].iftry;

	if(!iftry){
#if 0
        	rv = license_gen_feedback(feedback,&length,fbtype);
		if(rv !=0 )
			return rv;

        	if(strncmp(feedback,license->feedback->feedback,33)!=0)
                	return CS2C_LICENSE_FEEDBACK_ERR;
#endif
		if(fbtype==0) {	
			memcpy(hash32,license->feedback->feedback+1,32);
			rv = verify_mac_hash(hash32);
			if(rv != 0)
				return CS2C_LICENSE_FEEDBACK_ERR;
		} else {

        		rv = license_gen_feedback(feedback,&length,fbtype);
			if(rv !=0 )
				return rv;

        		if(strncmp(feedback,license->feedback->feedback,33)!=0)
                		return CS2C_LICENSE_FEEDBACK_ERR;

		}	
	}

        rv = rsa_verify(license->signature->sig,license->signature->length,(char *)license->control_info,license->header->control_info_length);

        return rv;
}

static int license_verify_cpunum(int cpunum_lic, int cpunum_probe)
{
	if (cpunum_lic == 99 || cpunum_probe == 99) {
		return 1;
	}
	if ((cpunum_probe > 2) && (cpunum_lic < 3)) {
		return 5;
	} else {
		return 0;
	}
}

static int license_verify_from_license_struct_v2(cs2c_license *license, int target_pid, int *mtflag)
{
        int rv = 0;
        char feedback[FB_STRING_LENGTH_V2];
        int length = FB_STRING_LENGTH_V2;
	int fbtype = 0;
	int iftry = 0;
	int product_number = 0;
	int i = 0;
	char hash32[32];

	memset(feedback,0,FB_STRING_LENGTH_V2);
	fbtype = license->feedback->feedback[0]-0x30;
	product_number = license->header->control_info_length/sizeof(cs2c_license_control_info);	

	for(i=0;i<product_number;i++)
		iftry |= license->control_info[i].iftry;

	if(!iftry){
#if 0
        	rv = license_gen_feedback(feedback,&length,fbtype);
		if(rv !=0 )
			return rv;

        	if(strncmp(feedback,license->feedback->feedback,33)!=0)
                	return CS2C_LICENSE_FEEDBACK_ERR;
#endif
		if(fbtype==0) {	
			//memcpy(hash32,license->feedback->feedback+1,32);
			memcpy(hash32,license->feedback->feedback+1, FB_MAC_FIRST_SECTION * 2);
                        memcpy(hash32 + FB_MAC_FIRST_SECTION * 2, 
                               license->feedback->feedback + 1 + FB_MAC_FIRST_SECTION * 2 + FB_MACHINE_LENGTH , 
                               (FB_MAC_SENCOND_SECTION - FB_MAC_FIRST_SECTION) * 2);
                        memcpy(hash32 + FB_MAC_SENCOND_SECTION * 2,
                               license->feedback->feedback + 1 + FB_MAC_SENCOND_SECTION * 2 + FB_MACHINE_LENGTH + FB_PRODUCTID_LENGTH,
                               (FB_LENGTH_V2 - FB_MAC_SENCOND_SECTION * 2));
			rv = verify_mac_hash(hash32);
			if(rv != 0)
				return CS2C_LICENSE_FEEDBACK_ERR;
			/* add check machine type & cpu number */
			int fb_getmachinetype = 0;
			//int fb_mactype_inlic = 
			char fb_mactype_inlic_str[FB_MACHINE_LENGTH + 1];
			memset(fb_mactype_inlic_str, 0, FB_MACHINE_LENGTH + 1);
			memcpy(fb_mactype_inlic_str, license->feedback->feedback + 1 + FB_MAC_FIRST_SECTION * 2, FB_MACHINE_LENGTH);
			int ret_getmac = get_client_machinetype(&fb_getmachinetype);
        		if (ret_getmac != 0) {
		                fb_getmachinetype = 99;
		        }
			int fb_mactype_inlic = atoi(fb_mactype_inlic_str);
			if(fb_getmachinetype != fb_mactype_inlic) {
			    if((fb_getmachinetype == 0) && (fb_mactype_inlic != 0)) {
				*mtflag = 2; //license为物理机，使用系统为虚拟机
			    } else if ((fb_getmachinetype != 0) && (fb_mactype_inlic == 0)) {
				*mtflag = 3; //license为虚拟机，使用系统为物理机
			    //} else if ((fb_getmachinetype != 0) && (fb_mactype_inlic != 0)) {
			    } else if ((fb_getmachinetype != 0) && (fb_mactype_inlic != 0)) {
				// *mtflag = 5; //license 中CPU数量与本机实际CPU数量不符。
				*mtflag = license_verify_cpunum(fb_mactype_inlic, fb_getmachinetype);
			    } else {
				*mtflag = 1;
			    }
				//printf("您的机器类型与license证书设定的机器类型不符。\n");
			}
		} else {
			return CS2C_LICENSE_FEEDBACK_ERR;
		}	
	}

        rv = rsa_verify(license->signature->sig,license->signature->length,(char *)license->control_info,license->header->control_info_length);

        return rv;
}

static int check_license_header_try(cs2c_license_header *header)
{
	
	if(header->header_length!=8)
		return CS2C_LICENSE_INVALID_FILE;
	if(header->control_info_length % 27 !=0)
		return CS2C_LICENSE_INVALID_FILE;
	//if(header->feedback_length!=131)
	/*if(header->feedback_length!= (97 + FB_STRING_LENGTH_V2))
		return CS2C_LICENSE_INVALID_FILE;*/
	if(header->verify_length!=129)
		return CS2C_LICENSE_INVALID_FILE;
	return 0;
}

static int check_license_header(cs2c_license_header *header)
{
	
	/*if(header->header_length!=8)
		return CS2C_LICENSE_INVALID_FILE;
	if(header->control_info_length % 27 !=0)
		return CS2C_LICENSE_INVALID_FILE;*/
	//if(header->feedback_length!=131)
	int rv = 0;
        rv = check_license_header_try(header);
	if (rv != 0) {
		return rv;
	}
	if(header->feedback_length!= (97 + FB_STRING_LENGTH_V2))
		return CS2C_LICENSE_INVALID_FILE;
	/*if(header->verify_length!=129)
		return CS2C_LICENSE_INVALID_FILE;*/
	return 0;
}

static int get_license_struct_from_base64(char* code, int clen, cs2c_license *license)
{
        int blen = 0;
        char *buf = malloc(clen);
	char *sig;
	int siglen = 0;
	int rv = 0;
        blen = clen;

        if(code==NULL || clen<=0)
                return CS2C_LICENSE_PARA_ERR;

        memset(buf,0,clen);
        debase64(code,clen,buf,&blen);

//a valid license file would not smaller than 290
	if(blen<295)
	{
		rv = CS2C_LICENSE_INVALID_FILE; 
		goto END;
	}

//        cs2c_license *license_tmp = malloc(sizeof(cs2c_license));
        cs2c_license_header *header = malloc(sizeof(cs2c_license_header));
        cs2c_license_signature *signature = malloc(sizeof(cs2c_license_signature));
	cs2c_license_feedback *fb = malloc(sizeof(cs2c_license_feedback));

        memcpy(header,buf,8);

	rv = check_license_header_try(header);
	if(rv !=0 )
		goto END;


        cs2c_license_control_info *control_info = malloc(header->control_info_length);
        memcpy(control_info,buf+header->header_length,header->control_info_length);
        memcpy(fb,buf+header->header_length+header->control_info_length,header->feedback_length);
        memcpy(signature,buf+header->header_length+header->control_info_length+header->feedback_length,1);

	siglen = signature->length;
	if(siglen==0)
	{
		rv =  CS2C_LICENSE_LICENSE_INFO_ERR;
		goto END;
	}

	sig = malloc(siglen);
	memset(sig,0,siglen);
	
	memcpy(sig,buf+header->header_length+header->control_info_length+header->feedback_length+1,header->verify_length-1);
	signature->sig = sig;

        license->header = header;
        license->control_info =control_info;
	license->feedback = fb;
        license->signature = signature;

	int iftry = 0;
	iftry |= license->control_info[0].iftry;
	if (!iftry){
		rv = check_license_header(header);
	}
END:
	if(buf)
		free(buf);
        return rv;
}

static int license_verify_from_hardisk_v2(char *license_file_name, cs2c_license_control_info *control_info, int *pnum, cs2c_license_feedback *fb, int target_pid, int *mtflag)
{
        int license_fd;
        char *buf=NULL;
        int len = 0;
        int rv = 0;

	*pnum = 0;
        cs2c_license *license = malloc(sizeof(cs2c_license));

	license->header=NULL;
	license->control_info=NULL;
	license->feedback=NULL;
	license->signature=NULL;

        rv = get_fsig(license_file_name,&license_fd,&len,0);
	if(rv != 0)
		goto END;

        buf = malloc(len);
        memset(buf,0,len);

        if((rv=pread(license_fd,buf,len,0)) <= 0){
        	rv = CS2C_LICENSE_FILE_ERR;
		free(buf);
		close(license_fd);
		return rv;
        }

        rv = get_license_struct_from_base64(buf,len,license);
        if(rv != 0)
                goto END;

//	print_license(license);
        rv = license_verify_from_license_struct_v2(license, target_pid, mtflag);
	if(rv == 0)
	{
		*pnum = license->header->control_info_length/sizeof(cs2c_license_control_info);
		memcpy(control_info, license->control_info, *pnum*sizeof(cs2c_license_control_info));
		memcpy(fb,license->feedback,sizeof(cs2c_license_feedback));
	}
END:
        free_license_struct(license);
        if(buf)
		free(buf);
	close(license_fd);
        return rv;
}

static int license_verify_from_hardisk(char *license_file_name, cs2c_license_control_info *control_info, int *pnum, cs2c_license_feedback *fb)
{
        int license_fd;
        char *buf=NULL;
        int len = 0;
        int rv = 0;

	*pnum = 0;
        cs2c_license *license = malloc(sizeof(cs2c_license));

	license->header=NULL;
	license->control_info=NULL;
	license->feedback=NULL;
	license->signature=NULL;

        rv = get_fsig(license_file_name,&license_fd,&len,0);
	if(rv != 0)
		goto END;

        buf = malloc(len);
        memset(buf,0,len);

        if((rv=pread(license_fd,buf,len,0)) <= 0){
        	rv = CS2C_LICENSE_FILE_ERR;
		free(buf);
		close(license_fd);
		return rv;
        }

        rv = get_license_struct_from_base64(buf,len,license);
        if(rv != 0)
                goto END;

//	print_license(license);
        rv = license_verify_from_license_struct(license);
	if(rv == 0)
	{
		*pnum = license->header->control_info_length/sizeof(cs2c_license_control_info);
		memcpy(control_info, license->control_info, *pnum*sizeof(cs2c_license_control_info));
		memcpy(fb,license->feedback,sizeof(cs2c_license_feedback));
	}
END:
        free_license_struct(license);
        if(buf)
		free(buf);
	close(license_fd);
        return rv;
}

#ifndef NOUSB
static int license_verify_from_usbkey(cs2c_license_control_info *control_info,int *pnum, cs2c_license_feedback *fb)
{
        char *buf;
        int len = 0;
        int rv = 0;

	*pnum = 0;
        cs2c_license *license = malloc(sizeof(cs2c_license));

	license->header=NULL;
	license->control_info=NULL;
	license->feedback=NULL;
	license->signature=NULL;

	rv = license_ukey_read_base64(DEFAULT_USER_NEW_PASSWD, DEFAULT_USER_NEW_PASSWD_LEN, \
                             NULL, &len);
	if(rv != 0 || len == 0)
	{
		rv =  CS2C_LICENSE_USBKEY_PKCSERR;
		goto END;
	}

        buf = malloc(len+1);
        memset(buf,0,len+1);

	rv = license_ukey_read_base64(DEFAULT_USER_NEW_PASSWD, DEFAULT_USER_NEW_PASSWD_LEN, \
                             buf, &len);
	if(rv != 0)
		goto END;

        rv = get_license_struct_from_base64(buf,len,license);
        if(rv != 0)
                goto END;

//      print_license(license);
        rv = license_verify_from_license_struct(license);
	if(rv == 0)
	{
		*pnum = license->header->control_info_length/sizeof(cs2c_license_control_info);
		memcpy(control_info, license->control_info, *pnum*sizeof(cs2c_license_control_info));	
		memcpy(fb,license->feedback,sizeof(cs2c_license_feedback));
	}
END:
        free_license_struct(license);
        free(buf);
        return rv;
}
#endif





static int set_license_install_date(cs2c_license *license,int pid)
{
        int seconds_now= time((time_t*)NULL);
        char now[12];
        cs2c_license_control_info *tmp;
        int pnum = 0;
        int i = 0;

        memset(now,0,12);
        sprintf(now,"%d",seconds_now);

        tmp = license->control_info;
        pnum = license->header->control_info_length/sizeof(cs2c_license_control_info);

        for(i=0;i<pnum;i++)
        {
                if(tmp->pid == pid)
                {
			if(tmp->installed[0]!=0)
				return CS2C_LICENSE_DOUBLE_INSTALLED;

			memcpy(tmp->installed,now,12);	
                        return 0;
                }
                tmp++;
        }

	return CS2C_LICENSE_NO_PID;
}

static int get_control_info_from_license(cs2c_license *license, cs2c_license_control_info *control_info, int pid)
{
	cs2c_license_control_info *tmp;
	int pnum = 0;
	int i = 0;

	tmp = license->control_info;
	pnum = license->header->control_info_length/sizeof(cs2c_license_control_info);

	for(i=0;i<pnum;i++)
	{
		if(tmp->pid == pid)
		{
			memcpy(control_info,tmp,sizeof(cs2c_license_control_info));
			return 0;
		}
		tmp++;
	}

	return CS2C_LICENSE_NO_PID;
}

#ifndef NOUSB
static int license_install_to_ukey(int pid)
{
        int rv = 0;
        char *buf=NULL;
        char sign[256];
        memset(sign,0,256);
	int slen = 256;
        int len = 0;

        cs2c_license *license = malloc(sizeof(cs2c_license));

	license->header=NULL;
	license->control_info=NULL;
	license->feedback=NULL;
	license->signature=NULL;

        rv = license_ukey_read_base64(DEFAULT_USER_NEW_PASSWD, DEFAULT_USER_NEW_PASSWD_LEN, \
                             NULL, &len);
        if(rv !=0 || len==0)
        {
                rv =  CS2C_LICENSE_USBKEY_PKCSERR;
                goto END;
        }

        buf = malloc(len+1);
        memset(buf,0,len+1);

        rv = license_ukey_read_base64(DEFAULT_USER_NEW_PASSWD, DEFAULT_USER_NEW_PASSWD_LEN, \
                             buf, &len);
        if(rv != 0)
                goto END;

        rv = get_license_struct_from_base64(buf,len,license);
        if(rv != 0)
                goto END;

	rv = set_license_install_date(license,pid);
	if(rv != 0)
		goto END;

        rv = rsa_sign((char *)license->control_info,license->header->control_info_length,sign,&slen);
        if(rv != 0){

                return CS2C_LICENSE_RSA_SIGN_FAILED;
        }

	license->signature->length = slen;
	memcpy(license->signature->sig,sign,slen);

	rv = license_gen_to_ukey(license);
//	print_license(license);
END:
	if(buf)
		free(buf);	

	if(license)
		free_license_struct(license);

        return rv;
}
#endif

static int  create_dir(const   char   *path)
{
        char   dirname[256];
        strcpy(dirname, path);
        int   i,len = strlen(dirname);

        if(dirname[len-1]!='/')
        	strcat(dirname, "/");

        len = strlen(dirname);

        for(i=1;i<len;i++)
        {
                if(dirname[i]=='/')
                {
                        dirname[i] = 0;
                        if(access(dirname, 0)!=0)
                        {
                                if(mkdir(dirname, 0755)==-1)
                                {
                                        return  -1;
                                }
                        }
                        dirname[i] = '/';
                }
        }

        return 0;
}



static int get_ifinstalled(char *md5sum)
{
        FILE *fh;
	int rv = 0;
        char line[128];

        memset(line,0,128);

        fh = fopen(DEFAULT_LICENSE_INSTALLED, "a+");
        if (!fh) {
                if(access("/etc/license", 0)!=0 || access("/etc/license/client",0)!=0)
                        rv = create_dir("/etc/license/client");
                if(rv==-1)
                {
                        return CS2C_LICENSE_FILE_ERR;

                }

                fh = fopen(DEFAULT_LICENSE_INSTALLED, "a+");

                if(!fh){
                        return CS2C_LICENSE_FILE_ERR;
                }
        }

        while(!feof(fh))
        {
                fgets(line,33,fh);
                if(!strncmp(line,md5sum,32))
                        return CS2C_LICENSE_DOUBLE_INSTALLED;
                memset(line,0,128);
        }

	fclose(fh);

	chmod(DEFAULT_LICENSE_INSTALLED,0666);

	return 0;
}


static int write_installed(char *md5sum)
{

        FILE *fh;

        fh = fopen(DEFAULT_LICENSE_INSTALLED, "a");
        if (!fh) {
		perror("fopen");
                return CS2C_LICENSE_FILE_ERR;
        }

	fputs(md5sum,fh);
	fputc('\n',fh);

	fclose(fh);

	chmod(DEFAULT_LICENSE_INSTALLED,0666);
	return 0;
}



static int license_install_to_hardisk(char * license_file_name,int pid)
{
        int rv = 0;
        int license_fd;
        char *buf;
        int len = 0;
        char sign[256];
	u8  md5sum[33];
	u8  md5[33];
	int slen = 256;
	int md5len=33;
	int md5sumlen=33;
	int i = 0;

        memset(sign,0,256);
        memset(md5sum,0,33);
        memset(md5,0,33);

        cs2c_license *license = malloc(sizeof(cs2c_license));

	license->header=NULL;
	license->control_info=NULL;
	license->feedback=NULL;
	license->signature=NULL;

        get_fsig(license_file_name,&license_fd,&len,1);
        buf = malloc(len);
        memset(buf,0,len);

        if((rv=pread(license_fd,buf,len,0)) <= 0){
		perror("pread");
		rv = CS2C_LICENSE_FILE_ERR; 
		free(buf);
		close(license_fd);
		return rv;
        }

	rv = mkmd5(buf,len,md5,&md5len);
//	printf("md5len = %d\n",md5len);
	if(rv!=0)
		goto END;

        for(i=0;i<md5len;i++)
                sprintf(md5sum+i*2,"%02x",md5[i]);
//	printf("%s\n",md5sum);

	rv = get_ifinstalled(md5sum);
	if(rv != 0)
		goto END;

        rv = get_license_struct_from_base64(buf,len,license);
        if(rv != 0)
                goto END;

        rv = set_license_install_date(license,pid);
        if(rv != 0)
                goto END;

        rv = rsa_sign((char *)license->control_info,license->header->control_info_length,sign,&slen);
        if(rv != 0){

                return CS2C_LICENSE_RSA_SIGN_FAILED;
        }

	license->signature->length = slen;
	memcpy(license->signature->sig,sign,slen);
        rv = license_gen_to_file(license,license_file_name);
//	print_license(license);
	
	if(rv !=0 )
		goto END;

	rv = write_installed(md5sum);

END:
	if(buf)
		free(buf);

	close(license_fd);
	if(license)
		free_license_struct(license);

        return rv;
}



static int get_license_install_date(cs2c_license *license, int pid, int *installed)
{
	cs2c_license_control_info *tmp;
	int i = 0;
	int pnum = 0;
        if(pid>255)
                return CS2C_LICENSE_PARA_ERR;

        pnum = license->header->control_info_length/sizeof(cs2c_license_control_info);
        tmp = license->control_info;
        for(i=0;i<pnum;i++)
        {
                if(tmp->pid == pid)
                {
			if(tmp->installed[0]==0)
				return CS2C_LICENSE_NEED_INSTALL;

                        *installed = strtoul(tmp->installed,NULL,strlen(tmp->installed));
                        return 0;
                }
                tmp++;
        }

        return CS2C_LICENSE_NO_PID;
	
}

static int get_license_days(cs2c_license *license, int pid, int *days)
{
	cs2c_license_control_info *tmp;
	int i =0;
	int pnum = 0;
	if(pid>255)
		return CS2C_LICENSE_PARA_ERR;

	pnum = license->header->control_info_length/sizeof(cs2c_license_control_info);
	tmp = license->control_info;
	for(i=0;i<pnum;i++)
	{
		if(tmp->pid == pid)
		{
			*days = tmp->days;
			return 0;
		}
		tmp++;
	}

	return CS2C_LICENSE_NO_PID;
}


#ifndef NOUSB
static int license_get_ifexpired_from_ukey(int pid,int *expired)
{
        int rv = 0;
        char *buf=NULL;
        int len = 0;
	int days = 0;
        int seconds_now= time((time_t*)NULL);
	int seconds_install = 0;

        cs2c_license *license = malloc(sizeof(cs2c_license));

	license->header=NULL;
	license->control_info=NULL;
	license->feedback=NULL;
	license->signature=NULL;

        rv = license_ukey_read_base64(DEFAULT_USER_NEW_PASSWD, DEFAULT_USER_NEW_PASSWD_LEN, \
                             NULL, &len);
        if(rv !=0 || len==0)
        {
                rv =  CS2C_LICENSE_USBKEY_PKCSERR;
                goto END;
        }

        buf = malloc(len+1);
        memset(buf,0,len+1);

        rv = license_ukey_read_base64(DEFAULT_USER_NEW_PASSWD, DEFAULT_USER_NEW_PASSWD_LEN, \
                             buf, &len);
        if(rv != 0)
                goto END;

        rv = get_license_struct_from_base64(buf,len,license);
        if(rv != 0)
                goto END;

        rv = get_license_install_date(license,pid,&seconds_install);
        if(rv != 0)
                goto END;

	rv = get_license_days(license,pid,&days);
        if(rv != 0)
                goto END;
#ifdef LICENSE_LIB_DEBUG
        printf("seconds_now is %d\n",seconds_now);
        printf("seconds_install is %d\n",seconds_install);
        printf("days is %d\n",days);
#endif
	if(((seconds_now-seconds_install)/(24*3600))>days)
	{
		*expired = 1;
//		rv = CS2C_LICENSE_IS_EXPIRED;
	}else *expired = 0;
END:
	
	if(buf)
		free(buf);

	return rv;
}
#endif


static int license_get_ifexpired_from_hardisk(char *license_file_name,int pid,int *expired)
{
        int rv = 0;
        char *buf=NULL;
	int license_fd;
        int len = 0;
        int days = 0;
        int seconds_now= time((time_t*)NULL);
        int seconds_install = 0;




        cs2c_license *license = malloc(sizeof(cs2c_license));

	license->header=NULL;
	license->control_info=NULL;
	license->feedback=NULL;
	license->signature=NULL;

        get_fsig(license_file_name,&license_fd,&len,0);
        buf = malloc(len);
        memset(buf,0,len);

        if((rv=pread(license_fd,buf,len,0)) <= 0){
		perror("pread");
        	rv = CS2C_LICENSE_FILE_ERR;
		free(buf);
		close(license_fd);
		return rv;
        }

        rv = get_license_struct_from_base64(buf,len,license);
        if(rv != 0)
                goto END;


        rv = get_license_install_date(license,pid,&seconds_install);
        if(rv != 0)
                goto END;

        rv = get_license_days(license,pid,&days);
        if(rv != 0)
                goto END;
#ifdef LICENSE_LIB_DEBUG
	printf("seconds_now is %d\n",seconds_now);
	printf("seconds_install is %d\n",seconds_install);
	printf("days is %d\n",days);
#endif
        if((seconds_now-seconds_install)>(days*24*3600))
        {
		*expired = 1;
//	        rv = CS2C_LICENSE_IS_EXPIRED;
	}else *expired = 0;
END:

        if(buf)
                free(buf);
	close(license_fd);

        return rv;
}


/*
* func:        generate a feedback code for license file which is a md5
* param:       fbtype can be 0,1,2 which means feedback is generated from mac,disk or ukey
* return:      0 is SUCCESS,non zero if FAILED
*/
int license_gen_feedback(char *feedback, int *length, int fbtype)
{
        u8 tmp[128];
	int tlen = 128;
        memset(tmp,'\0',128);
        u8 md5[16];
	int len = 16;
        int i = 0;
	int rv = 0;

#ifndef NOUSB
	if(fbtype == 1)
	{
		if(check_if_key_plugin()==0)
			return CS2C_LICENSE_NO_USBKEY;

		if(check_if_key_plugin()>1)
			return CS2c_LICENSE_MORE_USBKEY;

		if(get_sn_from_ukey(tmp,&tlen)!=0)
		{
#ifdef LICENSE_LIB_DEBUG
			printf("Failed to get UKEY Serial Number!\n");
#endif
			return CS2C_LICENSE_FEEDBACK_GEN_ERR;   
		}
	}else 
#endif 
	if(fbtype == 0){
        	if(fetch_mac(tmp, &tlen) != 0){
#ifdef LICENSE_LIB_DEBUG
                	printf("Failed to get MAC address! \n");
#endif
                	return CS2C_LICENSE_FEEDBACK_GEN_ERR;
        	}
	}else if(fbtype == 2){
		if(fetch_hardisk_info(tmp,&tlen) != 0){
#ifdef LICENSE_LIB_DEBUG
			printf("Failed to get Hardisk Info! \n");
#endif
			return CS2C_LICENSE_FEEDBACK_GEN_ERR;
		}
	}


        if(mkmd5(tmp,tlen,md5,&len)<0){
                return -1;
        }

	feedback[0]=0x30+fbtype;

        for(i=0;i<len;i++)
                sprintf(feedback+i*2+1,"%02x",md5[i]);

        *length=33;
//	printf("%s\n",feedback);
        return 0;
}


/*
* func:        generate a feedback code for license file which is a md5
* param:       fbtype can be 0,1,2 which means feedback is generated from mac,disk or ukey
* //param:       licversion: 1:original, 2: new version. default is 1, can call this without the param.
* param:       product_id: use it for version 2, default is 0.
* return:      0 is SUCCESS,non zero if FAILED
*/
int license_gen_feedback_v2(char *feedback, int *length, int fbtype, int product_id)
{
        u8 tmp[128];
	int tlen = 128;
        memset(tmp,'\0',128);
        u8 md5[16];
	int len = 16;
        int i = 0;
	int rv = 0;
        char fb_productid_map = 0;
        int fb_getmachinetype = 0;

	//if(licversion ==2 && product_id ==0) {
	if(product_id == 0) {
		return CS2C_LICENSE_FEEDBACK_GEN_ERR;
	}

#ifndef NOUSB
	if(fbtype == 1)
	{
		if(check_if_key_plugin()==0)
			return CS2C_LICENSE_NO_USBKEY;

		if(check_if_key_plugin()>1)
			return CS2c_LICENSE_MORE_USBKEY;

		if(get_sn_from_ukey(tmp,&tlen)!=0)
		{
#ifdef LICENSE_LIB_DEBUG
			printf("Failed to get UKEY Serial Number!\n");
#endif
			return CS2C_LICENSE_FEEDBACK_GEN_ERR;   
		}
	}else
#endif
	 if(fbtype == 0){
        	if(fetch_mac(tmp, &tlen) != 0){
#ifdef LICENSE_LIB_DEBUG
                	printf("Failed to get MAC address! \n");
#endif
                	return CS2C_LICENSE_FEEDBACK_GEN_ERR;
        	}
	}else if(fbtype == 2){
		if(fetch_hardisk_info(tmp,&tlen) != 0){
#ifdef LICENSE_LIB_DEBUG
			printf("Failed to get Hardisk Info! \n");
#endif
			return CS2C_LICENSE_FEEDBACK_GEN_ERR;
		}
	}


        if(mkmd5(tmp,tlen,md5,&len)<0){
                return -1;
        }

	/*1. mediatype fill to feedback*/
	feedback[0]=0x30+fbtype;

	/*2. MAC first section fill to feedback*/
        //for(i=0;i<len;i++)
        //        sprintf(feedback+i*2+1,"%02x",md5[i]);
        for (i = 0; i < FB_MAC_FIRST_SECTION; i++) {
		sprintf(feedback+i*2+1,"%02x",md5[i]);
	}

	/*3. add machine type*/
	/*3.1 get machine type*/

        int ret_getmac = get_client_machinetype(&fb_getmachinetype);
        if (ret_getmac != 0) {
		fb_getmachinetype = 99;
	}
	/*3.2 machine type fill to feedback*/
	sprintf(feedback + 1 + FB_MAC_FIRST_SECTION * 2, "%02d", fb_getmachinetype);

	/*4. MAC second section fill to feedback*/
	for (i = FB_MAC_FIRST_SECTION; i < FB_MAC_SENCOND_SECTION; i++) {
                sprintf(feedback+i*2+1+FB_MACHINE_LENGTH,"%02x",md5[i]);
        }
	/*5.add PID*/
	/* delete the process for one byte of pid.
 	* for (i=0; i < PID_LIST_LENGTH; i++) {
     		if (product_id == PRODUCTID_LIST[i]) {
			fb_productid_map = PRODUCTID_MAP_LIST[i];
       			break;
     		}
  	}
	if (fb_productid_map != 0) {
		feedback[] = fb_productid_map;
	}*/
	/*5.1 PID fille to feedback*/
        sprintf(feedback + 1 + FB_MAC_SENCOND_SECTION * 2 + FB_MACHINE_LENGTH, "%02x", product_id);
	
	/*6. MAC third section fill to feedback*/
	for (i = FB_MAC_SENCOND_SECTION; i < len; i++) {
                sprintf(feedback + i*2 + 1 + FB_MACHINE_LENGTH + FB_PRODUCTID_LENGTH,"%02x",md5[i]);
        }

        //*length=33;
        *length=37;
//	printf("%s\n",feedback);
        return 0;
}

/*
* func:        verify if the license is legal version2
* param:       license_file_name,if NULL,verify from ukey
* return:      0 is SUCCESS,non zero if FAILED
*/
int license_verify_v2(char *license_file_name, cs2c_license_control_info *control_info, int *pnum, cs2c_license_feedback *fb, int target_pid, int *mtflag)
{

        int     rv              = 0;
        int     ukey_flag       = 0;
        int     ukey_plugflag   = 0;

        if(license_file_name == NULL)
                ukey_flag = 1;

#ifdef NOUSB
	ukey_plugflag = 0;
#else
        ukey_plugflag = check_if_key_plugin();
#endif

        if(ukey_flag ==1 && ukey_plugflag == 0)
                return CS2C_LICENSE_NO_USBKEY;

#ifndef NOUSB
        if(ukey_flag == 1)
	{
                rv = license_verify_from_usbkey(control_info,pnum,fb);
        } else {
#endif
                rv = license_verify_from_hardisk_v2(license_file_name,control_info,pnum,fb,target_pid, mtflag);
                //rv = license_verify_from_hardisk(license_file_name,control_info,pnum,fb);
#ifndef NOUSB
	}
#endif
        return rv;
}

/*
* func:        verify if the license is legal
* param:       license_file_name,if NULL,verify from ukey
* return:      0 is SUCCESS,non zero if FAILED
*/
int license_verify(char *license_file_name, cs2c_license_control_info *control_info, int *pnum, cs2c_license_feedback *fb)
{

        int     rv              = 0;
        int     ukey_flag       = 0;
        int     ukey_plugflag   = 0;

        if(license_file_name == NULL)
                ukey_flag = 1;

#ifdef NOUSB
	ukey_plugflag = 0;
#else
        ukey_plugflag = check_if_key_plugin();
#endif

        if(ukey_flag ==1 && ukey_plugflag == 0)
                return CS2C_LICENSE_NO_USBKEY;

#ifndef NOUSB
        if(ukey_flag == 1)

                rv = license_verify_from_usbkey(control_info,pnum,fb);
        else
#endif
                rv = license_verify_from_hardisk(license_file_name,control_info,pnum,fb);

        return rv;
}


/*
* func:        intall a license file or a license ukey version2
* param:       if license_file_name == NULL, install it to ukey
* return:      0 is SUCCESS,non zero if FAILED
*/
int license_install_v2(char *license_file_name,int pid, int target_pid, int *mtflag)
{
        int rv = 0;
        int ukey_flag = 0;
	int pnum = 0;
	cs2c_license_control_info control_info[10];
	cs2c_license_feedback fb;

	//if(pid>256)
	if(pid>256 || target_pid>255)
		return CS2C_LICENSE_PARA_ERR;

        if(license_file_name == NULL)
                ukey_flag = 1;

	
        //rv = license_verify(license_file_name,(cs2c_license_control_info *)&control_info,&pnum,&fb);
        rv = license_verify_v2(license_file_name,(cs2c_license_control_info *)&control_info,&pnum,&fb, target_pid, mtflag);
        if(CS2C_LICENSE_OK != rv){

                write_log(LOG_LEVEL_0,"check_license",rv);
                return rv;
        }

#ifndef NOUSB
        if(ukey_flag == 1)

                rv = license_install_to_ukey(pid);
        else
#endif
                rv = license_install_to_hardisk(license_file_name,pid);

        return rv;

}

/*
* func:        intall a license file or a license ukey
* param:       if license_file_name == NULL, install it to ukey
* return:      0 is SUCCESS,non zero if FAILED
*/
int license_install(char *license_file_name,int pid)
{
        int rv = 0;
        int ukey_flag = 0;
	int pnum = 0;
	cs2c_license_control_info control_info[10];
	cs2c_license_feedback fb;

	if(pid>256)
		return CS2C_LICENSE_PARA_ERR;

        if(license_file_name == NULL)
                ukey_flag = 1;

	
        rv = license_verify(license_file_name,(cs2c_license_control_info *)&control_info,&pnum,&fb);
        if(CS2C_LICENSE_OK != rv){

                write_log(LOG_LEVEL_0,"check_license",rv);
                return rv;
        }

#ifndef NOUSB
        if(ukey_flag == 1)

                rv = license_install_to_ukey(pid);
        else
#endif
                rv = license_install_to_hardisk(license_file_name,pid);

        return rv;

}

/*
* func:        make sure a license file or a license key is not expired
* param:       if license_file_name == NULL, checking license ukey
* return:      0 is SUCCESS,non zero if FAILED
*/
int license_get_ifexpired(char *license_file_name, int pid, int *expired)
{
        int rv = 0;
        int ukey_flag = 0;
        int pnum = 0;
        cs2c_license_control_info control_info;
        cs2c_license_feedback fb;

        if(license_file_name == NULL)
                ukey_flag = 1;

#ifndef NOUSB
        if(ukey_flag == 1)

                rv = license_get_ifexpired_from_ukey(pid,expired);
        else
#endif
                rv = license_get_ifexpired_from_hardisk(license_file_name,pid,expired);


	return rv;
}


int license_get_lib_version(char *v, int *vlen)
{
	int rv = get_lib_version(v, vlen);
	return rv;
}

