#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <json/json.h>
#include <sys/timeb.h>
#include <dirent.h>

#include "license-server-web.h"
#include "consts.h"
#include "usbkey.h"
#include "log.h"
#include "rsa.h"

#define DEF_LICFILES_DIR "/opt/license_files/"
#define DEFAULT_LICENSE_CONFIG "/etc/license/licfiles.conf"
static int FEEDBACK_LENGTH = 33;
static char ret_filename[64];
static char def_licfiles_dir[80];


static char *trimWhitespace(char *str)
{
    char *end;
    while(isspace(*str)) str++;

    end = str + strlen(str) - 1;
    while(end > str && isspace(*end)) end--;

    *(end+1) = 0;

    return str;
}

int check_license_args(cs2c_license_control_info *control_info, int num, cs2c_license_feedback *feedback)
{
	int i = 0;
	int rv = 0;
	int iftry = 0;
	cs2c_license_control_info *control = control_info;	

	get_license_tryflag(control_info, num, &iftry);

	for(i = 0;i < rv;i++)
	{
		rv = check_license_control_info(control);
		if(rv != 0)
			return rv;

		control+=sizeof(cs2c_license_control_info);
	}


	rv = check_license_feedback(feedback,iftry);
	return rv;
}


int check_license_feedback(cs2c_license_feedback *feedback, int iftry)
{
	int rv = 0;
	int i = 0;
	
	if(feedback == NULL)
		return CS2C_LICENSE_STRUCT_ERR;

	if(iftry == 1)
	{
		for(i=1;i<FEEDBACK_LENGTH;i++)
			if(feedback->feedback[i]!=0)
				return CS2C_LICENSE_STRUCT_ERR;
	}else{
		for(i=0;i<FEEDBACK_LENGTH;i++)
			if(feedback->feedback[i]==0)
				return CS2C_LICENSE_FEEDBACK_ERR;
	}

	return CS2C_LICENSE_OK;
}

int check_license_control_info(cs2c_license_control_info *control_info)
{
	int rv = 0;
	int i = 0;

	if(control_info == NULL)
		return CS2C_LICENSE_PARA_ERR;

	
	return CS2C_LICENSE_OK;
}




int make_license_struct(cs2c_license_control_info *control_info,cs2c_license *license,int product_number,cs2c_license_feedback *fb)
{
	int rv = 0;
	int slen = 256;
	char sign[256];
	int i = 0;

	memset(sign,0,256);

	rv = rsa_sign((char *)control_info,product_number*sizeof(cs2c_license_control_info),sign,&slen);
	if(rv != 0){

		return CS2C_LICENSE_RSA_SIGN_FAILED;
	}

	cs2c_license_header *header = malloc(sizeof(cs2c_license_header));
	cs2c_license_control_info *control = malloc(product_number*sizeof(cs2c_license_control_info));
	cs2c_license_feedback *feedback = malloc(sizeof(cs2c_license_feedback));
	cs2c_license_signature *signature = malloc(sizeof(cs2c_license_signature));

	memcpy(control,control_info,product_number*sizeof(cs2c_license_control_info));
	memcpy(feedback,fb,sizeof(cs2c_license_feedback));

/*
	printf("%d\n",slen);
	for(i=0;i<slen;i++)
		printf("%02x",sign[i]);
*/
	signature->sig = malloc(slen);
	memcpy(signature->sig,sign,slen);
	signature->length = slen;

	header->header_length = 8;
	header->control_info_length = product_number*sizeof(cs2c_license_control_info);
	//header->feedback_length = sizeof(cs2c_license_feedback);
	header->feedback_length = 97 + FEEDBACK_LENGTH + 1;
	header->verify_length = 1+slen;

	license->header = header;
	license->control_info = control;
	license->feedback = feedback;
	license->signature = signature;
	
	return CS2C_LICENSE_OK;
}


int get_base64_from_license_struct(char* code, int *clen, cs2c_license *license)
{
	u8 *buf;
	int blen;
	int license_length = license->header->header_length + \
				license->header->control_info_length + \
				license->header->feedback_length + \
				license->header->verify_length;

	buf = malloc(license_length);
	blen = license_length;

	memcpy(buf,license->header,license->header->header_length);
	memcpy(buf+license->header->header_length,license->control_info,license->header->control_info_length);
	memcpy(buf+license->header->header_length+license->header->control_info_length,license->feedback,license->header->feedback_length);
	memcpy(buf+license->header->header_length+license->header->control_info_length+license->header->feedback_length,license->signature,1);
	memcpy(buf+license->header->header_length+license->header->control_info_length+license->header->feedback_length+1,license->signature->sig,license->header->verify_length-1);
	
	base64(buf,blen,code,clen);
	free(buf);

	return CS2C_LICENSE_OK;
}




int license_gen_to_file(cs2c_license *license, const char *license_file_name)
{
	int rv = 0;
	int license_fd;
	char buf[10240];
	int len = 10240;
	int i = 0;

	license_fd = open(license_file_name,O_RDWR|O_CREAT|O_TRUNC,0600);
	if(license_fd<0)
		return CS2C_LICENSE_FILE_ERR;

	rv = get_base64_from_license_struct(buf,&len,license);
        if(CS2C_LICENSE_OK != rv) {

                write_log(LOG_LEVEL_0,"get_base64_from_license_struct",rv);
                return rv;
        }
	
//	printf("%s\n",buf);
	
	if((rv=write(license_fd,buf,len))<0) {

		write_log(LOG_LEVEL_0,"license_gen_to_file",rv);
		close(license_fd);		
		return rv;
	}
	
	close(license_fd);
	chmod(license_file_name,0666);
	return CS2C_LICENSE_OK;
}

#ifndef NOUSB
int license_gen_to_ukey(cs2c_license *license)
{
        int rv = 0;
        char buf[1024];
        int len = 1024;
        int i = 0;

#ifdef	LOONGSON_PLATFORM
	return CS2C_LICENSE_MIPS_NO_UKEY;
#endif

        rv = get_base64_from_license_struct(buf,&len,license);
        if(CS2C_LICENSE_OK != rv) {

                write_log(LOG_LEVEL_0,"get_base64_from_license_struct",rv);
                return rv;
        }

	rv = license_ukey_setpin(DEFAULT_USER_NEW_PASSWD, DEFAULT_USER_NEW_PASSWD_LEN);
        if(CS2C_LICENSE_OK != rv) {

                write_log(LOG_LEVEL_0,"cense_ukey_setpin",rv);
                return rv;
        }

	
        rv = license_ukey_write_base64(DEFAULT_USER_NEW_PASSWD,DEFAULT_USER_NEW_PASSWD_LEN,buf,len);
	if(CS2C_LICENSE_OK != rv)
	{

                write_log(LOG_LEVEL_0,"license_ukey_write_base64",rv);
                return rv;
        }

        return 0;
}
#endif


int get_fsig(const char *licpath, int *fd, u32 *fsz,int flag)
{
        int ret;
        struct stat st;

	if(flag == 1)
        	*fd=open(licpath,O_RDWR);
	if(flag == 0)
		*fd=open(licpath,O_RDONLY);
        if(*fd<0){
		return CS2C_LICENSE_FILE_ERR;
	}
        ret=fstat(*fd,&st);
        if(ret<0){
                return ret;
        }
        *fsz=st.st_size;
        return 0;
}





int get_license_buffer(int license_fd, char *buf, int len)
{
	int ret = 0;
        if((ret=pread(license_fd,buf,len,0)) <= 0){
#ifdef LICENSE_LIB_DEBUG
                printf("Failed C\n");
#endif
                return CS2C_LICENSE_FILE_ERR;
        }
	return 0;
}



#if 0
int license_gen(cs2c_license_control_info *control_info,int pnum,cs2c_license_feedback *feedback)
{
	int rv = 0;
	int ukey_flag = 0;
	cs2c_license *license = malloc(sizeof(cs2c_license));

	license->header = NULL;
	license->control_info = NULL;
	license->feedback = NULL;
	license->signature = NULL;
	
//	print_license_control_info(control_info);	

	/* check if license struct is legal */
	rv = check_license_args(control_info,pnum,feedback);
	if(CS2C_LICENSE_OK != rv) {
		
		write_log(LOG_LEVEL_0,"check_license",rv);		
		return rv;
	}	
	
	rv = get_license_ukeyflag(feedback, &ukey_flag);
	if(CS2C_LICENSE_OK != rv){

		write_log(LOG_LEVEL_0,"get_license_ukeyflag",rv);		
		return rv;
	}

	rv = make_license_struct(control_info, license, pnum, feedback);
	if(CS2C_LICENSE_OK != rv){
		
		write_log(LOG_LEVEL_0,"make_license_struct",rv);		
		goto END;
	}

//	print_license(license);
	if(0 == ukey_flag)

		rv = license_gen_to_file(license,DEFAULT_LICENSE_NAME);
	else
		rv = license_gen_to_ukey(license);	

END:
	if(license)
		free_license_struct(license);

	return rv;
}
#endif

static int parsing_fbcode_v2(const char *fbstr, char *retstr, int *pidval, int *mtype, int *cpunum)
{
  if((fbstr == NULL) || (strlen(fbstr) != FEEDBACK_LENGTH) || (strncmp(fbstr, "0", 1) != 0)) {
    sprintf(retstr, "对不起，您输入的反馈码不正确。");
    return 8099;
  }
  
  char tmpstr[3];
  memset(tmpstr, 0, 3); 
  strncpy(tmpstr, fbstr + 1 + FB_MAC_SENCOND_SECTION *2 + FB_MACHINE_LENGTH, 2);
  sscanf(tmpstr, "%x", pidval);

  memset(tmpstr, 0, 3);
  strncpy(tmpstr, fbstr + 1 + FB_MAC_FIRST_SECTION * 2, 2); 
  
  *cpunum = atoi(tmpstr);

  if(*cpunum == 0) {
    *mtype = 0;
  } else {
    *mtype = 1;
  }
  return 0;
}

static int create_licfile(char *fbstr, u8 pid, char *retstr)
{
  cs2c_license_control_info control_info;
  cs2c_license_feedback fb_cs2clic;
  char license_file_name[256];
  int pnum = 1;

  memset(&control_info,0,sizeof(cs2c_license_control_info));
  memset(&fb_cs2clic,0,sizeof(cs2c_license_feedback));
  memset(license_file_name,0,256);

  sprintf(license_file_name, "%s/%s.dat", def_licfiles_dir, fbstr);
  //sprintf(license_file_name, "%s%s", DEF_LICFILES_DIR, fbstr);

  //control_info.pid = (u8)atoi(optarg);
  control_info.pid = pid;
  control_info.vid = 1;
  strcpy(fb_cs2clic.feedback,fbstr);

  //license_gen(&control_info,1,&fb);
  int rv = 0;
  int ukey_flag = 0;
  cs2c_license *license = malloc(sizeof(cs2c_license));
  license->header = NULL;
  license->control_info = NULL;
  license->feedback = NULL;
  license->signature = NULL;

  rv = check_license_args(&control_info,pnum,&fb_cs2clic);
  if(CS2C_LICENSE_OK != rv) {
    write_log(LOG_LEVEL_0,"check_license",rv);
    sprintf(retstr, "反馈码存在错误");
    rv = 8011;
    return rv;
  }

  rv = get_license_ukeyflag(fb_cs2clic, &ukey_flag);
  if(CS2C_LICENSE_OK != rv){
    write_log(LOG_LEVEL_0,"get_license_ukeyflag",rv);
    sprintf(retstr, "extract feedback error.");
    return rv;
  }

  rv = make_license_struct(&control_info, license, pnum, &fb_cs2clic);
  if(CS2C_LICENSE_OK != rv){
    write_log(LOG_LEVEL_0,"make_license_struct",rv);
    sprintf(retstr, "创建license结构体失败");
    rv = 8012;
    goto END;
  }
   
  if(0 == ukey_flag) {
    rv = license_gen_to_file(license,license_file_name);
    if (rv == 0) {
      sprintf(retstr, "%s", license_file_name);
    } else {
      rv = 1013;
      sprintf(retstr, "生成license文件失败");
    }
  }
#ifndef NOUSB 
  else {
    rv = license_gen_to_ukey(license);
  }
#endif

END:
  if(license)
    free_license_struct(license);

  return rv;
}

static struct json_object *setup_return_chkjobj_success(char *fbstr, int pidfb, int mtypefb, int cpunumfb)
{
  char *sucfbstr = fbstr;
  json_object *jstring_success_fb;
  json_object *jstring_pid, *jstring_mtype, *jstring_cpunum;
  json_object *sucobj = json_object_new_object();
  char pidfbstr[10];
  char mtypefbstr[10];
  char cpunumfbstr[3];
  memset(pidfbstr, 0, 10);
  memset(mtypefbstr, 0, 10);
  memset(cpunumfbstr, 0, 3);
  sprintf(pidfbstr, "%d", pidfb);
  sprintf(cpunumfbstr, "%d", cpunumfb);
  if (mtypefb) {
    sprintf(mtypefbstr, "物理机");
  } else {
    sprintf(mtypefbstr, "虚拟机");
  }

  if (sucfbstr) {
    jstring_success_fb = json_object_new_string(sucfbstr);
  } else {
    jstring_success_fb = json_object_new_string("");
  }
  jstring_pid = json_object_new_string(pidfbstr);
  jstring_mtype = json_object_new_string(mtypefbstr);
  jstring_cpunum = json_object_new_string(cpunumfbstr);
  json_object_object_add(sucobj,"feedback_code", jstring_success_fb);
  json_object_object_add(sucobj,"pid", jstring_pid);
  json_object_object_add(sucobj,"type", jstring_mtype);
  json_object_object_add(sucobj,"CPU", jstring_cpunum);

  return sucobj;
}

static struct json_object *setup_return_crtjobj_success(char *fbstr, char *filepath)
{
  char *sucfbstr = fbstr;
  char *sucfilepath = filepath;
  json_object *jstring_success_fb;
  json_object *jstring_success_filepath;
  json_object *sucobj = json_object_new_object();

  if (sucfbstr) {
    jstring_success_fb = json_object_new_string(sucfbstr);
  } else {
    jstring_success_fb = json_object_new_string("");
  }
  if (sucfilepath) {
    jstring_success_filepath = json_object_new_string(sucfilepath);
  } else {
    jstring_success_filepath = json_object_new_string("");
  }
  json_object_object_add(sucobj,"feedback_code", jstring_success_fb);
  json_object_object_add(sucobj,"filepath", jstring_success_filepath);

  return sucobj;
}

static struct json_object *setup_return_jobj_err(char *fbstr, int errcode, char *errdesc)
{
  char *failfbstr = fbstr;
  char *failerrdesc = errdesc;
  json_object *jstring_fail_fb;
  json_object * failobj = json_object_new_object();
  char errcodestr[65];
  memset(errcodestr, 0, 65);
  sprintf(errcodestr, "%d", errcode);

  if (failfbstr) {
    jstring_fail_fb = json_object_new_string(failfbstr);
  } else {
    jstring_fail_fb = json_object_new_string("");
  }
  json_object *jstring_fail_errcode = json_object_new_string(errcodestr);
  json_object *jstring_fail_errdesc = json_object_new_string(failerrdesc);
  json_object_object_add(failobj,"feedback_code", jstring_fail_fb);
  json_object_object_add(failobj,"err_code", jstring_fail_errcode);
  json_object_object_add(failobj,"err_desc", jstring_fail_errdesc);

  return failobj;
}

static int get_curtime_str(char *curtime, char *curtime_ms)
{
  time_t timep;
  struct tm *p;
  struct timeb tp;
  time(&timep);
  ftime(&tp);
  p = localtime(&timep);
  sprintf(curtime, "%d%d%d%02d%02d%02d", 1900 + p->tm_year, 1 + p->tm_mon, p->tm_mday,
               p->tm_hour,p->tm_min, p->tm_sec);
  sprintf(curtime_ms, "%s%03ld", curtime, tp.millitm);

  return 0;
}

static char * get_licfiles_defdir(char *dirpath)
{
  int linelength = 128;
  FILE *fh;
  char line[linelength];
  memset(line,0,linelength);
  char *strhead = "LICFILES_DEFAULT_DIR=";
  char *trimstr;
  fh = fopen(DEFAULT_LICENSE_CONFIG, "r");
  if (fh) {
    while(!feof(fh)){
      fgets(line,linelength,fh);
      trimstr = trimWhitespace(line);
      
      if (!strncmp(trimstr, strhead, strlen(strhead))) {
        strncpy(dirpath, trimstr + strlen(strhead), strlen(trimstr) - strlen(strhead));
	break;
      }
    }
    fclose(fh);
  }
  if(strlen(dirpath) == 0){
    sprintf(dirpath, "/opt/license_files/");
  }
  return dirpath;
}

static int CreateDir(const char *sPathName) 
  { 
  char   DirName[256]; 
  strcpy(DirName, sPathName); 
  int   i,len = strlen(DirName); 
  if(DirName[len-1]!='/') 
    strcat(DirName, "/"); 
  
  len = strlen(DirName); 
  for(i=1; i<len; i++) 
  { 
    if(DirName[i]=='/') 
    { 
      DirName[i] = 0; 
      if(opendir(DirName) == NULL) 
      { 
        if(mkdir(DirName, 0755)==-1) 
        {  
           printf("DirName:%s\n", DirName);
           return -1;  
        } 
      } 
      DirName[i] = '/'; 
    } 
  } 
  
  return 0; 
} 

static int set_licfiles_dir(const char *timestr)
{
  char def_dir[60];
  memset(def_dir,0,60);
  get_licfiles_defdir(def_dir);

  sprintf(def_licfiles_dir, "%s/%s/", def_dir, timestr);
  //printf("defdir:%s\n", def_licfiles_dir);

  int retval = CreateDir(def_licfiles_dir);
  return retval;
}

const char * create_license_files(const char *json_instring, int version_intf)
{
  //char *retobj_str = return_string;
  //char *retobj_str = NULL;
  //static char *ret_filename = NULL;
    
  char curtime_str[16];
  char curtime_ms_str[20];
  memset(curtime_str, 0, 16);
  memset(curtime_ms_str, 0, 20);
  get_curtime_str(curtime_str, curtime_ms_str);

  int json_parse_error = 0;
  json_object * retjobj = json_object_new_object();

  json_object *success_array = json_object_new_array();
  json_object_object_add(retjobj, "success", success_array);
  json_object *fail_array = json_object_new_array();
  json_object_object_add(retjobj,"fail", fail_array);

  json_object * successobj = json_object_new_object();
  json_object * failobj = json_object_new_object();

  json_object * jobj_crt_root = json_tokener_parse(json_instring);
  if (is_error(jobj_crt_root)) {
    failobj = setup_return_jobj_err(NULL, 9001, "解析传入的json字符串出错.");
    json_object_array_add(fail_array, failobj);
    json_parse_error = 1;
    goto ENDGEN;
  }

  if (version_intf == 1) {
    FEEDBACK_LENGTH = 33;
  } else if (version_intf == 2) {
    FEEDBACK_LENGTH = 37;
  } else {
      failobj = setup_return_jobj_err(NULL, 9002, "不支持此接口版本.");
      json_object_array_add(fail_array, failobj);
      json_parse_error = 1;
      goto ENDGEN;
  }

  int ret_setdir = set_licfiles_dir(curtime_str);
  if (ret_setdir) { /*setup dir failed*/
      failobj = setup_return_jobj_err(NULL, 1001, "创建存放license文件的目录出错.");
      json_object_array_add(fail_array, failobj);
      json_parse_error = 1;
      goto ENDGEN;
  }

  if (!json_object_is_type(jobj_crt_root, json_type_array)) {
    failobj = setup_return_jobj_err(NULL, 9003, "传入的json字符串非数组类型");
    json_object_array_add(fail_array, failobj);
    json_parse_error = 1;
    goto ENDGEN;
  }
  int fb_list_len =  json_object_array_length (jobj_crt_root);
  //printf("fb list length:%d\n", fb_list_len);
  int i;
  char fbstr[60];
  int pidval;
  int ret_crtlic;
  char retstr_crtlicfile[256];
  json_object *jobj_crt_one, *fbobj, *pidobj;
  for(i=0; i < json_object_array_length(jobj_crt_root); i++) {
    jobj_crt_one = json_object_array_get_idx(jobj_crt_root, i);
    if ((!json_object_object_get_ex(jobj_crt_one, "feedback_code", &fbobj)) ||
        (!json_object_is_type(fbobj, json_type_string))) {
        //failobj = setup_return_jobj_err(NULL, -2, "Sorry, get the feedback_code string error.");
        failobj = setup_return_jobj_err(NULL, 9004, "对不起，提取feedback_code字符串出错");
        json_object_array_add(fail_array, failobj);
        continue;
    }
    memset(fbstr, 0, 60);
    sprintf(fbstr, "%s", json_object_get_string(fbobj));

    if ((!json_object_object_get_ex(jobj_crt_one, "pid", &pidobj)) ||
        (!json_object_is_type(pidobj, json_type_string))) {
        //(!json_object_is_type(pidobj, json_type_int))) {
        //failobj = setup_return_jobj_err(fbstr, -2, "Sorry, get the PID error.");
        failobj = setup_return_jobj_err(fbstr, 9005, "对不起，提取PID 出错");
        json_object_array_add(fail_array, failobj);
	continue;
    }
    //pidval = json_object_get_int(pidobj);
    pidval = atoi(json_object_get_string(pidobj));

    memset(retstr_crtlicfile, 0, 256);
    ret_crtlic = create_licfile(fbstr, pidval, retstr_crtlicfile);

    if(ret_crtlic) { /*failed to create license file*/
        failobj = setup_return_jobj_err(fbstr, ret_crtlic, retstr_crtlicfile);
        json_object_array_add(fail_array, failobj);
    } else { /*success to create license file*/
        successobj = setup_return_crtjobj_success(fbstr, retstr_crtlicfile); 
        json_object_array_add(success_array, successobj);
    }
  }

ENDGEN:
  
  //retobj_str = json_object_to_json_string(retjobj);
  /*if (retstr_crtlicfile) {
      free(retstr_crtlicfile); }
  if (fbstr) {
      free(fbstr); }*/
  /*if (json_parse_error) {
    retobj_str = (char*)calloc(512, sizeof(char));
  } else {
    retobj_str = (char*)calloc(fb_list_len * 512,sizeof(char));
  }
  sprintf(retobj_str, "%s", json_object_to_json_string(retjobj));*/

  sprintf(ret_filename, "/var/cache/license_data_%s", curtime_ms_str);
  json_object_to_file(ret_filename, retjobj);
  
  /* free json objects */
  if (retjobj) {
     json_object_put(retjobj);
  }
  if (jobj_crt_root) {
     json_object_put(jobj_crt_root);
  }

  return ret_filename;
  //return retobj_str;

}

const char * feedback_code_check(const char * codelist, int version_intf)
{
  char curtime_str[16];
  char curtime_ms_str[20];
  memset(curtime_str, 0, 16);
  memset(curtime_ms_str, 0, 20);
  get_curtime_str(curtime_str, curtime_ms_str);

  int json_parse_error = 0;
  json_object * retjobj = json_object_new_object();

  json_object *success_array = json_object_new_array();
  json_object_object_add(retjobj, "success", success_array);
  json_object *fail_array = json_object_new_array();
  json_object_object_add(retjobj,"fail", fail_array);

  json_object * successobj = json_object_new_object();
  json_object * failobj = json_object_new_object();

  json_object * jobj_chk_root = json_tokener_parse(codelist);
  if (is_error(jobj_chk_root)) {
    failobj = setup_return_jobj_err(NULL, 9001, "解析传入的json字符串出错.");
    json_object_array_add(fail_array, failobj);
    json_parse_error = 1;
    goto ENDCHK;
  }

  if (version_intf == 1) {
    FEEDBACK_LENGTH = 33;
  } else if (version_intf == 2) {
    FEEDBACK_LENGTH = 37;
  } else {
      failobj = setup_return_jobj_err(NULL, 9002, "不支持此接口版本.");
      json_object_array_add(fail_array, failobj);
      json_parse_error = 1;
      goto ENDCHK;
  }

  if (!json_object_is_type(jobj_chk_root, json_type_array)) {
    failobj = setup_return_jobj_err(NULL, 9003, "传入的json字符串非数组类型");
    json_object_array_add(fail_array, failobj);
    json_parse_error = 1;
    goto ENDCHK;
  }
  int fb_list_len =  json_object_array_length (jobj_chk_root);
  int i;
  char fbstr[60];
  int pidval_fb = 0, cpunum_fb = 1, machinetype_fb = 1;
  int ret_chklic;
  char retstr_chklicfile[256];
  json_object *jobj_chk_one, *fbobj, *pidobj;
  for(i=0; i < json_object_array_length(jobj_chk_root); i++) {
    jobj_chk_one = json_object_array_get_idx(jobj_chk_root, i);
    if ((!json_object_object_get_ex(jobj_chk_one, "feedback_code", &fbobj)) ||
        (!json_object_is_type(fbobj, json_type_string))) { 
        failobj = setup_return_jobj_err(NULL, 9004, "对不起，提取feedback_code字符串出错");
        json_object_array_add(fail_array, failobj);
        continue;
    }
    memset(fbstr, 0, 60);
    sprintf(fbstr, "%s", json_object_get_string(fbobj));

    memset(retstr_chklicfile, 0, 256);
    ret_chklic = parsing_fbcode_v2(fbstr, retstr_chklicfile, &pidval_fb, &machinetype_fb, &cpunum_fb);

    if(ret_chklic) { /*failed to create license file*/
        failobj = setup_return_jobj_err(fbstr, ret_chklic, retstr_chklicfile);
        json_object_array_add(fail_array, failobj);
    } else { /*success to create license file*/
        successobj = setup_return_chkjobj_success(fbstr, pidval_fb, machinetype_fb, cpunum_fb);
        json_object_array_add(success_array, successobj);
    }
  }

ENDCHK:

  sprintf(ret_filename, "/var/cache/check_code_%s", curtime_ms_str);
  json_object_to_file(ret_filename, retjobj);

  /* free json objects */
  if (retjobj) {
     json_object_put(retjobj);
  }
  if (jobj_chk_root) {
     json_object_put(jobj_chk_root);
  }

  return ret_filename;
}

