#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <json/json.h>
#include <sys/timeb.h>
#include <dirent.h>

#include "license-server.h"
#include "consts.h"
#include "usbkey.h"
#include "log.h"
#include "rsa.h"

#define DEF_LICFILES_DIR "/opt/license_files/"
#define DEFAULT_LICENSE_CONFIG "/etc/license/licfiles.conf"
static int FEEDBACK_LENGTH = 37;
#if 0
static char ret_filename[64];
static char def_licfiles_dir[80];
#endif

static char *trimWhitespace(char *str)
{
    char *end;
    while(isspace(*str)) str++;

    end = str + strlen(str) - 1;
    while(end > str && isspace(*end)) end--;

    *(end+1) = 0;

    return str;
}

int check_license_args(cs2c_license_control_info *control_info, int num, cs2c_license_feedback *feedback)
{
	int i = 0;
	int rv = 0;
	int iftry = 0;
	cs2c_license_control_info *control = control_info;	

	get_license_tryflag(control_info, num, &iftry);

	for(i = 0;i < rv;i++)
	{
		rv = check_license_control_info(control);
		if(rv != 0)
			return rv;

		control+=sizeof(cs2c_license_control_info);
	}


	rv = check_license_feedback(feedback,iftry);
	return rv;
}


int check_license_feedback(cs2c_license_feedback *feedback, int iftry)
{
	int rv = 0;
	int i = 0;
	
	if(feedback == NULL)
		return CS2C_LICENSE_STRUCT_ERR;

	if(iftry == 1)
	{
		for(i=1;i<FEEDBACK_LENGTH;i++)
			if(feedback->feedback[i]!=0)
				return CS2C_LICENSE_STRUCT_ERR;
	}else{
		for(i=0;i<FEEDBACK_LENGTH;i++)
			if(feedback->feedback[i]==0)
				return CS2C_LICENSE_FEEDBACK_ERR;
	}

	return CS2C_LICENSE_OK;
}

int check_license_control_info(cs2c_license_control_info *control_info)
{
	int rv = 0;
	int i = 0;

	if(control_info == NULL)
		return CS2C_LICENSE_PARA_ERR;

	
	return CS2C_LICENSE_OK;
}




int make_license_struct(cs2c_license_control_info *control_info,cs2c_license *license,int product_number,cs2c_license_feedback *fb)
{
	int rv = 0;
	int slen = 256;
	char sign[256];
	int i = 0;

	memset(sign,0,256);

	rv = rsa_sign((char *)control_info,product_number*sizeof(cs2c_license_control_info),sign,&slen);
	if(rv != 0){

		return CS2C_LICENSE_RSA_SIGN_FAILED;
	}

	cs2c_license_header *header = malloc(sizeof(cs2c_license_header));
	cs2c_license_control_info *control = malloc(product_number*sizeof(cs2c_license_control_info));
	cs2c_license_feedback *feedback = malloc(sizeof(cs2c_license_feedback));
	cs2c_license_signature *signature = malloc(sizeof(cs2c_license_signature));

	memcpy(control,control_info,product_number*sizeof(cs2c_license_control_info));
	memcpy(feedback,fb,sizeof(cs2c_license_feedback));

/*
	printf("%d\n",slen);
	for(i=0;i<slen;i++)
		printf("%02x",sign[i]);
*/
	signature->sig = malloc(slen);
	memcpy(signature->sig,sign,slen);
	signature->length = slen;

	header->header_length = 8;
	header->control_info_length = product_number*sizeof(cs2c_license_control_info);
	//header->feedback_length = sizeof(cs2c_license_feedback);
	header->feedback_length = 97 + FEEDBACK_LENGTH + 1;
	header->verify_length = 1+slen;

	license->header = header;
	license->control_info = control;
	license->feedback = feedback;
	license->signature = signature;
	
	return CS2C_LICENSE_OK;
}


int get_base64_from_license_struct(char* code, int *clen, cs2c_license *license)
{
	u8 *buf;
	int blen;
	int license_length = license->header->header_length + \
				license->header->control_info_length + \
				license->header->feedback_length + \
				license->header->verify_length;

	buf = malloc(license_length);
	blen = license_length;

	memcpy(buf,license->header,license->header->header_length);
	memcpy(buf+license->header->header_length,license->control_info,license->header->control_info_length);
	memcpy(buf+license->header->header_length+license->header->control_info_length,license->feedback,license->header->feedback_length);
	memcpy(buf+license->header->header_length+license->header->control_info_length+license->header->feedback_length,license->signature,1);
	memcpy(buf+license->header->header_length+license->header->control_info_length+license->header->feedback_length+1,license->signature->sig,license->header->verify_length-1);
	
	base64(buf,blen,code,clen);
	free(buf);

	return CS2C_LICENSE_OK;
}




int license_gen_to_file(cs2c_license *license, const char *license_file_name)
{
	int rv = 0;
	int license_fd;
	char buf[10240];
	int len = 10240;
	int i = 0;

	license_fd = open(license_file_name,O_RDWR|O_CREAT|O_TRUNC,0600);
	if(license_fd<0)
		return CS2C_LICENSE_FILE_ERR;

	rv = get_base64_from_license_struct(buf,&len,license);
        if(CS2C_LICENSE_OK != rv) {

                write_log(LOG_LEVEL_0,"get_base64_from_license_struct",rv);
                return rv;
        }
	
//	printf("%s\n",buf);
	
	if((rv=write(license_fd,buf,len))<0) {

		write_log(LOG_LEVEL_0,"license_gen_to_file",rv);
		close(license_fd);		
		return rv;
	}
	
	close(license_fd);
	chmod(license_file_name,0666);
	return CS2C_LICENSE_OK;
}

#ifndef NOUSB
int license_gen_to_ukey(cs2c_license *license)
{
        int rv = 0;
        char buf[1024];
        int len = 1024;
        int i = 0;

#ifdef	LOONGSON_PLATFORM
	return CS2C_LICENSE_MIPS_NO_UKEY;
#endif

        rv = get_base64_from_license_struct(buf,&len,license);
        if(CS2C_LICENSE_OK != rv) {

                write_log(LOG_LEVEL_0,"get_base64_from_license_struct",rv);
                return rv;
        }

	rv = license_ukey_setpin(DEFAULT_USER_NEW_PASSWD, DEFAULT_USER_NEW_PASSWD_LEN);
        if(CS2C_LICENSE_OK != rv) {

                write_log(LOG_LEVEL_0,"cense_ukey_setpin",rv);
                return rv;
        }

	
        rv = license_ukey_write_base64(DEFAULT_USER_NEW_PASSWD,DEFAULT_USER_NEW_PASSWD_LEN,buf,len);
	if(CS2C_LICENSE_OK != rv)
	{

                write_log(LOG_LEVEL_0,"license_ukey_write_base64",rv);
                return rv;
        }

        return 0;
}
#endif


int get_fsig(const char *licpath, int *fd, u32 *fsz,int flag)
{
        int ret;
        struct stat st;

	if(flag == 1)
        	*fd=open(licpath,O_RDWR);
	if(flag == 0)
		*fd=open(licpath,O_RDONLY);
        if(*fd<0){
		return CS2C_LICENSE_FILE_ERR;
	}
        ret=fstat(*fd,&st);
        if(ret<0){
                return ret;
        }
        *fsz=st.st_size;
        return 0;
}





int get_license_buffer(int license_fd, char *buf, int len)
{
	int ret = 0;
        if((ret=pread(license_fd,buf,len,0)) <= 0){
#ifdef LICENSE_LIB_DEBUG
                printf("Failed C\n");
#endif
                return CS2C_LICENSE_FILE_ERR;
        }
	return 0;
}




int license_gen(cs2c_license_control_info *control_info,int pnum,cs2c_license_feedback *feedback)
{
	int rv = 0;
	int ukey_flag = 0;
	cs2c_license *license = malloc(sizeof(cs2c_license));

	license->header = NULL;
	license->control_info = NULL;
	license->feedback = NULL;
	license->signature = NULL;
	
//	print_license_control_info(control_info);	

	/* set FEEDBACK_LENGTH according to feedback string */
	if (feedback->feedback != NULL) {
		FEEDBACK_LENGTH = strlen(feedback->feedback);
	}

	/* check if license struct is legal */
	rv = check_license_args(control_info,pnum,feedback);
	if(CS2C_LICENSE_OK != rv) {
		
		write_log(LOG_LEVEL_0,"check_license",rv);		
		return rv;
	}	
	
	rv = get_license_ukeyflag(feedback, &ukey_flag);
	if(CS2C_LICENSE_OK != rv){

		write_log(LOG_LEVEL_0,"get_license_ukeyflag",rv);		
		return rv;
	}

	rv = make_license_struct(control_info, license, pnum, feedback);
	if(CS2C_LICENSE_OK != rv){
		
		write_log(LOG_LEVEL_0,"make_license_struct",rv);		
		goto END;
	}

//	print_license(license);
	if(0 == ukey_flag)

		rv = license_gen_to_file(license,DEFAULT_LICENSE_NAME);
#ifndef NOUSB
	else
		rv = license_gen_to_ukey(license);	
#endif

END:
	if(license)
		free_license_struct(license);

	return rv;
}

