#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <dirent.h>
#include <sys/stat.h>
#include "license-base.h"
#include <linux/types.h>
#include <linux/ethtool.h>
#include <linux/sockios.h>

#define _PATH_PROCNET_DEV             "/proc/net/dev"
#define _PATH_PROCBOND_DIR            "/proc/net/bonding/"
#define new(p) ((p) = xmalloc(sizeof(*(p))))
# define isspace(c)     __isctype((c), _ISspace)

#ifndef MAX_ADDR_LEN
#define MAX_ADDR_LEN   32
#endif

struct nic {

        struct nic *next, *prev;
        char name[IFNAMSIZ];
        char hwaddr[32];
	int index;
};


static struct nic *int_list, *int_last;

void *xmalloc(unsigned sz)
{
        void *p = calloc(sz,1);
        if (!p) errno=ENOMEM, err("xmalloc");
        return p;
}

/* Like strncpy but make sure the resulting string is always 0 terminated. */
char *safe_strncpy(char *dst, const char *src, size_t size)
{
	dst[size-1] = '\0';
	return strncpy(dst,src,size-1);
}

/* like strcmp(), but knows about numbers */
int nstrcmp(const char *astr, const char *b)
{
	const char *a = astr;

	while (*a == *b) {
		if (*a == '\0') return 0;
		a++;
		b++;
	}

	if (isdigit(*a)) {
		if (!isdigit(*b))
			return 1;
		while (a > astr) {
			a--;
			if (!isdigit(*a)) {
				a++;
				break;
			}

			if (!isdigit(*b)) return -1;
				b--;
        	}

        	return atoi(a) > atoi(b) ? 1 : -1;
	}

	return *a - *b;
}

/*NeoKylin add begin*/
/*add 20160408*/
char *trimWhitespace(char *str)
{
    char *end;
    /* Trim leading space */
    while(isspace(*str)) str++;
 
    /* Trim trailing space */
    end = str + strlen(str) - 1;
    while(end > str && isspace(*end)) end--;
    
    /* Write new null terminator */
    *(end+1) = 0;
   
    return str;
}

char hex2bin(char c)
{
    return (isdigit(c) ? (c)-'0': ((toupper(c))-'A')+10)  & 0xf;
}

int hexToBinary(char *hex, char *bin, int binLen)
{
    /* read from hex into bin, up to max binLen chars, return number written */
    char *h = hex;
    char *b = bin;
    char c;
    int i = 0;

    while((c = *h++) != '\0') {
      if(isxdigit(c)) {
        char val = hex2bin(c);
        if(isxdigit(*h)) {
          c = *h++;
          val = (val << 4) | hex2bin(c);
        }
        *b++ = val;
        if(++i >= binLen) return i;
      }
      else if(c != '.' &&
              c != '-' &&
              c != ':') { /* allow a variety of byte-separators */
        return i;
      }
    }
    return i;
}

static void upd_bond_slavemac(char *devname, char *devmac)
{
        struct nic *tmp;
        tmp = int_list;

        /* if there is no usefull nic, return error */
        if(tmp == NULL)
                return ;

        while(tmp!=NULL)
        {
                if (nstrcmp(tmp->name, devname) == 0) {
                    memset(tmp->hwaddr,0,32);
                    memcpy(tmp->hwaddr, devmac, 6);
                }
                tmp=tmp->next;
        }

        return ;
}

static void scan_bondmac(char *bonddev){
    char procFileName[256];
    snprintf(procFileName, 256, "/proc/net/bonding/%s", bonddev);
    FILE *procFile = fopen(procFileName, "r");
    if(procFile) {
        #define MAX_PROC_LINE_CHARS 240
        char line[MAX_PROC_LINE_CHARS];
        char slave_devname[16];
        char slave_devmac[6];

        while(fgets(line, MAX_PROC_LINE_CHARS, procFile)) {
            char buf_var[MAX_PROC_LINE_CHARS];
            char buf_val[MAX_PROC_LINE_CHARS];
            /* buf_var is up to first ':', buf_val is the rest */
            if(sscanf(line, "%[^:]:%[^\n]", buf_var, buf_val) == 2) {
                char *tok_var = trimWhitespace(buf_var);
                char *tok_val = trimWhitespace(buf_val);
                if(nstrcmp(tok_var, "Slave Interface") == 0) {
                   memset(slave_devname, 0, 16);
                   sprintf(slave_devname, "%s", tok_val);
                }
                if(slave_devname){
                    if(nstrcmp(tok_var, "Permanent HW addr") == 0) {
                       memset(slave_devmac,0,6);
                       if(hexToBinary(tok_val, slave_devmac, 6) == 6) {
#ifdef LICENSE_LIB_DEBUG
                          printf("%s: permanent HW addr :%s\n", slave_devname, slave_devmac);
#endif
                          /*todo: update orig mac address of devname*/
                          upd_bond_slavemac(slave_devname, slave_devmac);
                       }else{
#ifdef LICENSE_LIB_DEBUG
                          printf("mac transfer binary error.\n");
#endif
                       }
                    }
                }
            }
        }
        fclose(procFile);
    }
}

static void reset_bondmac(){
  DIR *dp;
  struct dirent *entry;

  if((dp = opendir(_PATH_PROCBOND_DIR)) == NULL) 
    {
        return;
    }

  while((entry = readdir(dp)) != NULL)
  {
    //if(strcmp(entry->d_name,".") != 0 && strcmp(entry->d_name, "..") != 0) {
    if(entry->d_type != DT_DIR) {
      scan_bondmac(entry->d_name);
    }
  }
  closedir(dp);
  return;
}

/*NeoKylin end*/

static struct nic * add_nic(char *name)
{
	struct nic *ife, **nextp, *new;

	for (ife = int_last; ife; ife = ife->prev) {
		int n = nstrcmp(ife->name, name);
		if (n == 0)
			return ife;
		if (n < 0)
			break;
	}

	new(new);
	safe_strncpy(new->name, name, IFNAMSIZ);
	nextp = ife ? &ife->next : &int_list;
	new->prev = ife;
	new->next = *nextp;
	if (new->next)
		new->next->prev = new;
	else
		int_last = new;
	*nextp = new;

	return new;
}

static int free_nic_list()
{
	struct nic *tmp;
	struct nic *next;

	tmp = int_list;
	for(tmp;tmp!=NULL;)
	{
		next=tmp->next;
		free(tmp);
		tmp=next;
	}	
	

	int_list = NULL;	
	int_last = NULL;
	
	return 0;
}
static int get_nic_from_socket()
{
	register int fd, intrface, retn = 0;
	struct ifreq buf[16];
	struct ifconf ifc;

	if ((fd = socket (AF_INET, SOCK_DGRAM, 0)) >= 0) {
		ifc.ifc_len = sizeof buf;
		ifc.ifc_buf = (caddr_t) buf;
		if (!ioctl (fd, SIOCGIFCONF, (char *) &ifc)) {

			intrface = ifc.ifc_len / sizeof (struct ifreq);
			// printf("interface num is intrface=%d\n\n\n",intrface);

			while (intrface-- > 0) {
			    add_nic(buf[intrface].ifr_name);
			//    printf("socket name :%s\n",buf[intrface].ifr_name);
			}

		} else { 
			return CS2C_LICENSE_SOCKETERR;
		}
	}
	
	close(fd);
	return 0;
}

static char *get_name(char *name, char *p)
{
	while (isspace(*p))
		p++;

	while (*p) {
		if (isspace(*p))
			break;
		if (*p == ':') {        /* could be an alias */
			char *dot = p, *dotname = name;
			*name++ = *p++;

			while (isdigit(*p))
				*name++ = *p++;

			if (*p != ':') {    /* it wasn't, backup */
				p = dot;
				name = dotname;
			}

			if (*p == '\0')
				return NULL;

			p++;
			break;
		}

		*name++ = *p++;
	}

	*name++ = '\0';
	return p;
}


static int get_nic_from_proc()
{
	FILE *fh;
	char buf[512];
	struct nic *ife;
	int err = 0;


	fh = fopen(_PATH_PROCNET_DEV, "r");
	if (!fh) {
		return CS2C_LICENSE_PROC_READERR;
	}

	fgets(buf, sizeof buf, fh); /* eat line */
	fgets(buf, sizeof buf, fh);


	while (fgets(buf, sizeof buf, fh)) {
		char *s, name[IFNAMSIZ];
		//printf("%s\n",buf);
		s = get_name(name, buf);
		ife = add_nic(name);
		//printf("proc name:%s\n",name);
	}

/*
    if (ferror(fh)) {
        perror(_PATH_PROCNET_DEV);
        err = -1;
    }
*/
	fclose(fh);

	return err;


}

static int get_all_nic_name()
{
	int rv = 0;
	rv = get_nic_from_socket();
	if(0!=rv)
		return rv;

	rv = get_nic_from_proc();
	if(0!=rv)
		return rv;

        return 0;
}


static int biosdevname(char *name, char *biosname,int *namelen)
{
	FILE *fp;
	int status;
	char path[1035];
	char cmd[128];
	int i = 0;

	memset(cmd,0,128);
	memset(path,0,1035);

	sprintf(cmd,"%s%s","biosdevname -i ",name);

	/* Open the command for reading. */
	fp=popen(cmd,"r");
	if (fp == NULL) {
		return -1;	
	}

	/* Read the output a line at a time - output it. */
	while (fgets(path, sizeof(path)-1, fp) != NULL) {
		if(path[0]!=0)
		{
			if(*namelen<strlen(path))
			{
				*namelen=strlen(path);
				return CS2C_LICENSE_BUF_SMALL;
			}

			for(i=0;i<strlen(path);i++)
				if(path[i]=='\n')
				{
					path[i] = '\0';
					break;
				}

			strcpy(biosname,path);	
			*namelen=strlen(path);
		}
	}

	/* close */
	pclose(fp);

	return 0;
}

static int delete_node(struct nic * node)
{

	if(node == int_list)
	{
		if(node->next != NULL)
			node->next->prev = NULL;
		else 
			int_last = NULL;
		int_list = node->next;
		free(node);
		return 0;
	}

	if(node == int_last)
	{
		if(node->prev != NULL)
			node->prev->next = NULL;
		else
			int_list = NULL;

		int_last = node->prev;
		free(node);
		return 0;
	}	

	node->prev->next = node->next;
	node->next->prev = node->prev;
	free(node);
	return 0;
}

static int remove_useless_nic()
{

	struct nic *tmp;
	struct nic *next;
	int rflag = 0;
	int rv = 0;

	tmp = int_list;

	/* if there is no usefull nic, return error */	
	if(tmp == NULL)
		return CS2C_LICENSE_NO_USEFUL_NIC;
 
	while(tmp!=NULL)
	{
		next = tmp->next;

		if(	!strncmp(tmp->name,"vmnet",5) 	|| 
			!strncmp(tmp->name,"vir",3) 	|| 
			!strncmp(tmp->name,"wlan",4) 	|| 
			!strncmp(tmp->name,"bond",4) 	||
			!strncmp(tmp->name,"br",2) 	|| 
			!strncmp(tmp->name,"lo",2)	||  
			!strncmp(tmp->name,"vnet",4)||
			strstr(tmp->name, "vdsm")||
			strstr(tmp->name, "virt")
			)
		{
			rflag = 1;
#ifdef LICENSE_LIB_DEBUG
			printf("%s ",tmp->name);
#endif
			delete_node(tmp);
		}
		tmp = next;
	}

#ifdef LICENSE_LIB_DEBUG
	if(rflag == 1) printf("were removed from list!\n");
#endif
	tmp = int_list;
	/* if there is no usefull nic, return error */	
	if(tmp == NULL)
		return CS2C_LICENSE_NO_USEFUL_NIC; 

#ifdef LICENSE_LIB_DEBUG
	printf("usefull nic:");
#endif
	while(tmp!=NULL)
	{
#ifdef LICENSE_LIB_DEBUG
		printf("%s ",tmp->name);
#endif
		tmp=tmp->next;
	}
#ifdef LICENSE_LIB_DEBUG
	printf("\n");
#endif
	return 0;

}

/*parameter:isver: 0: get mac for generate feedback code
 *                      1: get mac for verify license.
*/
//static int get_mac_from_nic_struct(struct nic *nic)
static int get_mac_from_nic_struct(struct nic *nic, int isver)
{
	int skfd = -1;
    	struct ifreq ifr;

	if(nic == NULL)
		return -1;

	skfd = socket(AF_INET, SOCK_DGRAM, 0);
    	if(skfd == -1)
    	{
        	return CS2C_LICENSE_SOCKETERR;
    	}

    	strcpy(ifr.ifr_name, nic->name);

	if (isver) {
		struct ethtool_perm_addr *epaddr;
		epaddr = malloc(sizeof(struct ethtool_perm_addr) + MAX_ADDR_LEN);
		epaddr->cmd = ETHTOOL_GPERMADDR;
		epaddr->size = MAX_ADDR_LEN;
		ifr.ifr_data = (void *)epaddr;
		if(ioctl(skfd, SIOCETHTOOL, &ifr) < 0)
		{
#ifdef LICENSE_LIB_DEBUG
	            	printf("%s\n",strerror(errno));
#endif
        	    	return CS2C_LICENSE_SOCKETERR;
	    	} else {
			memset(nic->hwaddr,0,32);
			memcpy(nic->hwaddr, epaddr->data, 6);
#ifdef LICENSE_LIB_DEBUG
			printf("nicname: %s, hwaddr: ", ifr.ifr_name);
			int i;
			for (i = 0; i < epaddr->size; i++) 
				printf("%c%02x", (i == 0) ? ' ' : ':', epaddr->data[i]);
			printf("\n");
#endif
		}
	} else { /*generate feedback code*/
    		if(ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0)
	    	{
#ifdef LICENSE_LIB_DEBUG
        	    	printf("%s\n",strerror(errno));
#endif
            		return CS2C_LICENSE_SOCKETERR;
	    	} else {
			memset(nic->hwaddr,0,32);
	        	memcpy(nic->hwaddr, ifr.ifr_hwaddr.sa_data, 6);
#ifdef LICENSE_LIB_DEBUG
			int i;
			for (i = 0; i < 6; i++)
				printf("%c%02x", (i == 0) ? ' ' : ':', nic->hwaddr[i]&0xff);
			printf("\n");
#endif
    		}
	}

    	close(skfd);	

	return 0;
}


/*parameter:isver:  0: get mac for generate feedback code
 *                  1: get mac for verify license.
*/
static int get_all_mac(int isver)
{

	int rv = 0;
	struct nic *tmp;
	tmp = int_list;

	/* if there is no usefull nic, return error */	
	if(tmp == NULL)
		return CS2C_LICENSE_NO_USEFUL_NIC;

	while(tmp!=NULL)
	{	
		/* tmp shouldn't be NULL */
                
		rv = get_mac_from_nic_struct(tmp, isver);
		if(rv != 0)
			return CS2C_LICENSE_NO_USEFUL_NIC;
		tmp=tmp->next;
	}

	/*add: if all of interfaces' permanent HW addr are 000000000 or fffffffff*/
	/*     then reset mac with socket. */
	tmp = int_list;
	if (isver) {
	    char macaddr[13];
	    int flag_macerr = 0;
	    while (tmp != NULL)
	    {
		memset(macaddr, 0, 13);
		sprintf(macaddr, "%02X%02X%02X%02X%02X%02X",tmp->hwaddr[0]&0xff,
                                tmp->hwaddr[1]&0xff,tmp->hwaddr[2]&0xff,tmp->hwaddr[3]&0xff,tmp->hwaddr[4]&0xff,tmp->hwaddr[5]&0xff);
		if (!strncmp(macaddr, "000000000000", 12) || !strncmp(macaddr, "ffffffffffff", 12) || !strncmp(macaddr, "FFFFFFFFFFFF", 12)) {
			flag_macerr = 1;
		} else {
			flag_macerr = 0;
			break;
		}
		tmp = tmp->next;
	    }
	    if (flag_macerr) {
		tmp = int_list;
		while(tmp!=NULL){
			rv = get_mac_from_nic_struct(tmp, 0); // use 0 for get mac form socket
	                if(rv != 0)
        	                return CS2C_LICENSE_NO_USEFUL_NIC;
                	tmp=tmp->next;
		}
	    }
	}

	return 0;
}

static int set_nic_list_index()
{
	
	struct nic *tmp;
	int index = 0;
	for(tmp=int_list;tmp!=NULL;tmp=tmp->next)
	{
		tmp->index = index;
		index++;	
	}

	return 0;
}

static int get_last_index()
{
	if(int_last!=NULL)
		return int_last->index;
	else
		return -1;
}


static struct nic * get_node_from_index(int index)
{

	struct nic * tmp;
	for(tmp=int_list;tmp!=NULL;tmp=tmp->next)
	{
		if(tmp->index==index)
			return tmp;
	}

	return NULL;
}

static int switch_node(int i, int j)
{

	if(i == j) return 0;

	struct nic * inode;	
	struct nic * jnode;
	struct nic * tmp1;
	struct nic * tmp2;

	inode = get_node_from_index(i);		
	jnode = get_node_from_index(j);		

	tmp1 = inode->next;
	tmp2 = inode->prev;

	if(inode->prev)
		inode->prev->next = jnode;
	if(jnode->prev)
		jnode->prev->next = inode;
	if(inode->next)
		inode->next->prev = jnode;
	if(jnode->next)
		jnode->next->prev = inode;


	if(jnode->next!=inode)
		inode->next = jnode->next;
	else
		inode->next = jnode;
	if(jnode->prev!=inode)
		inode->prev = jnode->prev;
	else
		inode->prev = jnode;
	if(tmp1!=jnode)
		jnode->next = tmp1;
	else
		jnode->next = inode;
	if(tmp2!=jnode)
		jnode->prev = tmp2;
	else
		jnode->prev = inode;

	inode->index = j;
	jnode->index = i;

	if(int_list == inode)	
		int_list = jnode;
	else if(int_list == jnode)
		int_list = inode;

	if(int_last == inode)	
		int_last = jnode;
	else if(int_last == jnode)	
		int_last = inode;
		
	return 0;

}

/* sort the nic list by mac address.
 * from smaller to bigger.
 */
static void quicksort_nic_list(int l,int h)
{

	struct nic *lnode;
	struct nic *hnode;
	struct nic *tmp;
	char keymac[6];
	int i,j;
	struct nic keynode;
	int rv = 0;

	memset(keymac,0,6);

	lnode = get_node_from_index(l);
	hnode = get_node_from_index(h);

	i = l;
	j = h;
	
	if(lnode==NULL || hnode==NULL)
		return;

	memcpy(keymac,lnode->hwaddr,6);
	memcpy(&keynode,lnode,sizeof(struct nic));

	while(i<j)
	{
		while(i<j && memcmp(hnode->hwaddr, keymac, 6)>0) 
		{
			j--;
			hnode = get_node_from_index(j);
		}

		if(i<j)
		{
			switch_node(i, j);
			i++;
			lnode = get_node_from_index(i);
		}

		while(i<j && strncmp(lnode->hwaddr,keymac, 6)<0)
		{
			i++;
			lnode = get_node_from_index(i);
		}

		if(i<j)
		{
			switch_node(i, j);
			j--;	
		}
	}

	if(l<i-1)
		quicksort_nic_list(l,i-1);			
	if(i+1<h)
		quicksort_nic_list(i+1,h);			

		
}


static int mach_nic_list_hash(char *hash32)
{

	struct nic *tmp;
	unsigned char md5[16];
	char mac_hash32[32];
	int len = 16;
	int i =0;

	memset(md5,0,16);
	memset(mac_hash32,0,32);

	for(tmp=int_list;tmp!=NULL;tmp=tmp->next)	
	{
#ifdef LICENSE_LIB_DEBUG
            printf("***choosen embedded nic: %s, ",tmp->name);
            printf("***mac is %02X:%02X:%02X:%02X:%02X:%02X\n",tmp->hwaddr[0]&0xff,
				tmp->hwaddr[1]&0xff,tmp->hwaddr[2]&0xff,tmp->hwaddr[3]&0xff,tmp->hwaddr[4]&0xff,tmp->hwaddr[5]&0xff);
#endif
		if(mkmd5(tmp->hwaddr,6,md5,&len)<0)
			return -1;
        	for(i=0;i<len;i++)
                	sprintf(mac_hash32+i*2,"%02x",md5[i]);
		if(memcmp(hash32,mac_hash32,32)==0)
			return 0;
	}

	return -1;
}

/* the strategy of final mac: 
 * 1.embedded NIC's MAC
 * 2.hex sorting of all MACS,for changing name of NIC.
 */
static int get_final_mac(char *mac, int *size)
{
	struct nic *tmp;
	int index = 0;
	int value = 0;
	int em_value = 0;
	int last_index = 0;
	int rv = 0;

	set_nic_list_index();
	last_index = get_last_index();
	if(last_index == -1)
		return CS2C_LICENSE_NO_USEFUL_NIC;

	quicksort_nic_list(0,last_index);

	for(tmp=int_list;tmp!=NULL;tmp=tmp->next)	
	{
		if(!strncmp(tmp->name,"em",2))	
		{
			memset(mac,0,*size);
			memcpy(mac,tmp->hwaddr,6);
			*size = 6;
#ifdef LICENSE_LIB_DEBUG
			printf("choosen embedded nic: %s, ",tmp->name);
			printf("mac is %02X:%02X:%02X:%02X:%02X:%02X\n",mac[0]&0xff,
				mac[1]&0xff,mac[2]&0xff,mac[3]&0xff,mac[4]&0xff,mac[5]&0xff);
#endif
			return 0;
		}
		
	}
	
	memset(mac,0,*size);
        memcpy(mac,int_list->hwaddr,6);
        *size = 6;
#ifdef LICENSE_LIB_DEBUG
        printf("choosen pci nic: %s, ",int_list->name);
        printf("mac is %02X:%02X:%02X:%02X:%02X:%02X\n",mac[0]&0xff,
                mac[1]&0xff,mac[2]&0xff,mac[3]&0xff,mac[4]&0xff,mac[5]&0xff);
#endif
        return 0;	

}


static int get_bios_nic_name()
{
	struct nic *tmp;
	struct nic *tmp1;
	tmp = int_list;

	/* if there is no usefull nic, return error */	
	if(tmp == NULL)
		return CS2C_LICENSE_NO_USEFUL_NIC;

	while(tmp!=NULL)	
	{
		
                char biosname[16];
                int  namelen = 16;
		memset(biosname, 0, 16);
                biosdevname(tmp->name,biosname,&namelen);
                if(biosname[0]!=0){
		      memset(tmp->name,0,IFNAMSIZ);
                      strncpy(tmp->name,biosname,namelen);
                }
                tmp=tmp->next;
	}
	
	return 0;
}



int fetch_mac(char *mac,int *size)
{
	struct ifreq ifr;
	int fd;
	char ifname[16];
	int rv = 0;
	int skfd = -1;
	int i = 0;

	memset(ifname,0,16);

	free_nic_list();
	rv = get_all_nic_name();
	if(rv != 0 ) return rv;

	rv = remove_useless_nic();	
	if(rv != 0 ) return rv;

	rv = get_all_mac(0);
	if(rv != 0 ) return rv;

#ifndef LOONGSON_PLATFORM    
	rv = get_bios_nic_name();
	if(rv != 0 ) return rv;

#endif
	rv = get_final_mac((unsigned char *)mac,size);
	return rv;
}


int verify_mac_hash(char *hash32)
{
	int rv = 0;

	free_nic_list();
	rv = get_all_nic_name();
	if(rv != 0 ) return rv;

	rv = remove_useless_nic();	
	if(rv != 0 ) return rv;

	rv = get_all_mac(1);
	if(rv != 0 ) return rv;

        /*NeoKylin add begin 20160411*/
        //reset_bondmac();
        /*NeoKylin end*/

#ifndef LOONGSON_PLATFORM    
	rv = get_bios_nic_name();
	if(rv != 0 ) return rv;
#endif
	rv = mach_nic_list_hash(hash32);

	return rv;
}
